//
// C++ Implementation: models
//
// Description:
//
//
// Author: fmt <fmt@localhost.localdomain>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <QtGui>
#include <QMessageBox>
#include <QProcess>
#include "default.h"
#include "document.h"
#include "mainwindow.h"
#include "projectwizard.h"
#include "models.h"
#include "records.h"
//#include "projectwizard.h"
#include "editline.h"

bool hasModelFiles(QString directory,QStringList &list,int &nummodel) {
    QDir dir(directory);
    if ( !dir.exists () ) {
        QMessageBox::warning ( 0, mainWindowCaption(),
                               QString ( "Directory <%1> does not exist." )
                               .arg (directory));
        return false;
    }
    QStringList filters;
    QString tmp = "";
    tmp = "\"model\" files";
    filters << "*" + modelFilterLocator();

    dir.setNameFilters(filters);
    QStringList listtemp;
    listtemp = dir.entryList(filters,QDir::Files);
    if (listtemp.isEmpty ()) {

        QMessageBox::warning ( 0,mainWindowCaption(),
                               QString ( "Directory <%1> \n does not contain %2." )
                               .arg ( dir.absolutePath () )
                               .arg ( tmp ) );
        nummodel = 0;
        return true;
    }

    foreach(QString str,listtemp) {
        str = directory + str;
        list << str;
    }
    nummodel = list.size();
    return true;

}
///---------------------------------------------------------------------------------------------
ModelsPageImpl::ModelsPageImpl(QWidget *parent,MDocument* doc,ProWizard::DocumentUsing documentUsing)
        : QDialog(parent),doc(doc),documentUsing(documentUsing) {
    setFont( parent->font() );
    setWindowIcon(QIcon(":/images/icon.png"));
    setupUi(this);
    InitDialog();
    SetDocumentToDialog();
}
bool ModelsPageImpl::CheckDialogParameeters() {

    return true;
}
void ModelsPageImpl::FinishWizard() {
    GetDocumentFromDialog();
}

void ModelsPageImpl::InitDialog() {
    lineEditModelDirectory->setReadOnly ( true );
    listWidgetStation->setSelectionMode(QAbstractItemView::MultiSelection);
    listWidgetModel->setSelectionMode(QAbstractItemView::SingleSelection);
    listWidgetStationModel->setSelectionMode(QAbstractItemView::NoSelection);

    connect( pushButtonModelDirectory, SIGNAL( clicked() ), this, SLOT(browse()));
    connect( pushButton3SmacStation, SIGNAL( clicked() ), this, SLOT(get3smacmodel()));
    connect( pushButtonSourceModel, SIGNAL( clicked() ), this, SLOT(getsourcemodel()));

    connect( listWidgetModel, SIGNAL( itemDoubleClicked ( QListWidgetItem * ) ), this, SLOT( doubleClickModelList(QListWidgetItem *) ) );

	//if (documentUsing == ProWizard::ProjectEdit) pushButtonModelDirectory->hide();

//    SetStationLists();
//    SetModelStationLists();

}
void ModelsPageImpl::doubleClickModelList(QListWidgetItem *item) {

    QItemSelectionModel * selmod = listWidgetStation->selectionModel ();
    QItemSelection sel = selmod->selection ();
    if (sel.count () == 0) return;
    QString modName = item->data ( Qt::DisplayRole).toString();
//QMessageBox::warning ( 0,"","doubleClickModelList");

    QItemSelectionRange range;
    for (int i = 0; i < sel.count (); ++i) {
        range = sel.at(i);
        int selTopRow    =   range.top();
        int selNumRows   =   range.height();
        for (int j = 0; j < selNumRows; ++j) {
            int row = selTopRow + j;
            QListWidgetItem * itemSta = listWidgetStation->item (row );
            QString str = itemSta->data ( Qt::DisplayRole).toString();

            QListWidgetItem * itemStaMod = listWidgetStationModel->item (row );
            QString str1 = modName + "  ";
            str1 += str;
            itemStaMod->setData (Qt::DisplayRole, str1);
            /*
            			listWidgetStationModel->count ();
            			for (int k = 0; k < listWidgetStationModel->count (); ++k) {
            				QListWidgetItem * itemStaMod = listWidgetStationModel->item (k );
            				QString str1 = itemStaMod->data ( Qt::DisplayRole).toString();
            				QStringList list = str.split(' ',QString::SkipEmptyParts);
            				if (str == list.at(1)) {
            					QString str2 = modName + "  ";
            					str2 += str;
            					itemStaMod->setData (Qt::DisplayRole, str2);
            					break;
            				}


            			}
            */
        }
    }
    listWidgetStation->clearSelection ();

    CheckExistModlInlistStationModel ();
}
void ModelsPageImpl::SetDocumentToDialog() {
//    foreach (LineDocProperty *docprop, propDocLine) docprop->SetValueFromDoc();

    if (doc->GetSourceModelType()->GetValue() == "0") radioButton3Smac->setChecked (true);
    else  radioButtonModelList->setChecked (true);
    labelSourceModel->setText(doc->GetNameSourceModel()->GetValue());

    QString dir = doc->GetModelNameDirectory()->GetValue();
    if (dir != QString()) SetAllLists(dir);



}
void ModelsPageImpl::GetDocumentFromDialog() {
    if (radioButton3Smac->isChecked () ) doc->GetSourceModelType()->SetValue("0");
    else doc->GetSourceModelType()->SetValue("1");
    doc->GetNameSourceModel()->SetValue(labelSourceModel->text());
    doc->GetModelNameDirectory()->SetValue(ModelDirectoryName);
    QStringList temp;
    for ( int i = 0; i < listWidgetStationModel->count (); ++i)
        temp << listWidgetStationModel->item (i)->data ( Qt::DisplayRole).toString();
    doc->GetStationModelList()->SetListToValue(temp);
    QMultiMap<QString,QString > map;
    for (int i = 0; i < modelList.size();++i) {
        QString mod = modelList.at(i);
        QListWidgetItem * item = listWidgetModel->item (i);
        QString str = item->data ( Qt::DisplayRole).toString();
        map.insert(str, mod);
    }
    doc->GetModelMap() = map;
}
void ModelsPageImpl::SetAllLists(QString &dir) {
    modelList.clear();
    int numModel = 0;
    if (!hasModelFiles(dir,modelList,numModel)) {
        ModelDirectoryName = QString();
        lineEditModelDirectory->setText(QString());
        return;
    }
    lineEditModelDirectory->setText(dir);
    ModelDirectoryName = dir;

    if (listWidgetModel->count () > 0) listWidgetModel->clear();

    for (int i = 0; i < modelList.size();++i) {
        QString mod = modelList.at(i);
        mod = MainWindow::self->strippedName(mod);
        mod = mod.remove(mod.lastIndexOf('.'),4);
        listWidgetModel->addItem (mod);
    }
    labelModel->setText(QString("Models: %1").arg(listWidgetModel->count()));
}
void ModelsPageImpl::browse() {

    QString dir = QFileDialog::getExistingDirectory(this, tr("Open Model Directory"),
                  MainWindow::self->openModelDirectory,
                  // QFileDialog::ShowDirsOnly |
                  QFileDialog::DontResolveSymlinks);
    if (!dir.isEmpty()) {
        MainWindow::self->openModelDirectory = dir;
        SetAllLists(dir);
    }

}

void ModelsPageImpl::SetModelStationLists() {
    if (listWidgetStationModel->count () > 0) listWidgetStationModel->clear();
    if (doc->GetStationModelList()->GetValue() == QString()) {
        QStringList lst;
        for (int i = 0; i < listWidgetStation->count (); ++i) {
            QListWidgetItem * item = listWidgetStation->item (i);
            QString str = item->data ( Qt::DisplayRole).toString();
            QString str1 = "mod?  " + str;
            listWidgetStationModel->addItem(str1);
        }
    } else {
        QStringList lst = doc->GetStationModelList()->GetList();
        listWidgetStationModel->addItems(lst);
    }

    labelStationModel->setText(QString("Stations: %1").arg(listWidgetStationModel->count()));
    CheckExistModlInlistStationModel ();
}
void ModelsPageImpl::SetStationLists() {
    if (listWidgetStation->count () > 0) listWidgetStation->clear();
    QListWidget* prorec = ProWizard::wizardSelf->recordsPage->recordsImpl->GetListWidgetProject ();
    QMap<QString, int> map;
    for (int i = 0; i < prorec->count (); ++i) {
        QListWidgetItem * itemProj = prorec->item (i);
        QString str = itemProj->data ( Qt::DisplayRole).toString();
        str = str.left(str.indexOf(':'));
        map.insert(str, i);
    }
    QList<QString> keys = map.keys();
    QString lst;
    for (int i = 0; i < keys.size (); ++i) {
        QString str = keys.at(i);
        listWidgetStation->addItem(str);
    }

    labelStation->setText(QString("Stations: %1").arg(listWidgetStation->count()));
}

bool ModelsPageImpl::CheckExistModlInlistStationModel () {
    bool find = true;
    for (int i = 0; i < listWidgetStationModel->count (); ++i) {
        QListWidgetItem * item = listWidgetStationModel->item (i);
        QString str = item->data ( Qt::DisplayRole).toString();
        if (str.contains('?')) {
            find = false;
            item->setForeground ( QBrush (Qt::blue) );
        } else
            item->setForeground ( QBrush (Qt::black) );
    }
    return find;
}

void ModelsPageImpl::get3smacmodel() {
    for (int i = 0; i < listWidgetStationModel->count (); ++i) {
        QListWidgetItem * item = listWidgetStationModel->item (i);
        QString str = item->data ( Qt::DisplayRole).toString();
        QStringList list = str.split(' ',QString::SkipEmptyParts);
        str = "3Smac " + list.at(1);
        item->setData (Qt::DisplayRole, str);
        item->setForeground ( QBrush (Qt::black) );
    }
}
void ModelsPageImpl::getsourcemodel() {
    if (radioButton3Smac->isChecked () )
        labelSourceModel->setText("3Smac");
    else {
        QList<QListWidgetItem *> lstItems = listWidgetModel->selectedItems ();
        if (lstItems.size() == 0) {
            QMessageBox::warning ( 0, mainWindowCaption(),
                                   QString ( "Select needed model name from Models list." ));
            return;
        }
        QListWidgetItem * item = lstItems.at(0);
        QString tmp = item->data ( Qt::DisplayRole).toString();
        labelSourceModel->setText(tmp);
    }
}



