#include <QtCore>
#include <QMessageBox>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <sys/ioctl.h>
#include <rpc/rpc.h>
#include <rpc/xdr.h>
#include "sei.h"
#include "ahhead.h"
extern "C"
{
	ahhed  getahhead ( FILE * );
	int getahdata ( FILE *file_in,ahhed head,char *array );
	int xdr_putrecord ( ahhed	*head,char *array, XDR	*xdrs );
	extern int  azidmod_ ( float *,float *,float *,float *,float *,float *,float *,float *,float * );
}
double time_ahead ( int y, int  m, int d,int h,int mn,float sc );
void time_back ( double time, int *y,int *m, int *d,int *h,int *mn,float *sc );
/*=======================================================================*/
_MLT::~_MLT() {;};
_MLT::_MLT ( int Init_n_sei,QStringList &list,int indDetrend ) : _SET_SEI ( Init_n_sei )
{
	SEISMO *ps;
	QString dirName = QFileInfo ( list.at ( 0 ) ).absoluteDir ().absolutePath ();
	char *str = dirName.toAscii ().data();
// Suppose 	in_dir and out_dir	not be used  !!!!!
	Set_in_dir ( str );
	Set_out_dir ( str );
	seiFilesGood = true;
	for ( int i = 0; i < Init_n_sei; i++ )
	{
		ps = Get_S ( i );
		QString fname = list.at ( i );
		ps->Set_fname ( fname );
		if ( !ps->Read_Header_Sei ( list.at ( i ) ,1,indDetrend ) )
		{
			seiFilesGood = false;
			return;
		}
// QMessageBox::critical(0,"Error",
//                         QString("point %1 of %2 '%3'").arg(i+1).arg(Init_n_sei).arg(list.at(i)));

	}
}
//========================================================================

_SET_SEI::_SET_SEI ( int n )
{

	I_N_SEI =      N_SEI = n;

	S = new SEISMO * [I_N_SEI];
	CS = new SEISMO * [I_N_SEI];


	for ( int i = 0; i < I_N_SEI; i++ )
	{
		S[i] = new SEISMO;
		CS[i] = S[i];
	}
}

_SET_SEI::~_SET_SEI()
{
	for ( int i = 0; i < I_N_SEI; i++ )  delete  S[i];
	delete [] S;
};

/*=======================================================================*/
void  _SET_SEI::Recalc_parametrs()
{
	int i;
	float tt0;
	int llag=0,mmax=0;
	long nnsampl;
	float ddt;
	S[0]->Get_t0 ( tt0 );
	S[0]->Get_lag ( llag );
	S[0]->Get_nsampl ( nnsampl );
	S[0]->Get_max ( mmax );
	S[0]->Get_dt ( ddt );
	time_start  = tt0 ;
	time_finish = tt0   + ( float ) nnsampl * ddt;
	max_nsample = nnsampl;
	common_max  = mmax;

	for ( i = 1;i < I_N_SEI ; i++ )
	{
		S[i]->Get_t0 ( tt0 );
		S[i]->Get_lag ( llag );
		S[i]->Get_nsampl ( nnsampl );
		S[i]->Get_max ( mmax );
		S[i]->Get_dt ( ddt );

		if ( tt0    < time_start )
			time_start = tt0 ;
		if ( ( tt0  + ( float ) nnsampl* ddt ) > time_finish )
			time_finish =  tt0 + ( float ) nnsampl*ddt;
		if ( nnsampl > max_nsample ) max_nsample =   nnsampl;

		if ( mmax > common_max ) common_max =  mmax;
	}
	common_time = time_finish- time_start;

};

/*=======================================================================*/


SEISMO::SEISMO() :  HEADER_SEI() {set = 0;}

SEISMO::~SEISMO() { if ( set !=0 ) delete [] set; set = 0;}

/*=======================================================================*/

HEADER_SEI::HEADER_SEI() {; }

HEADER_SEI::~HEADER_SEI() {;}
/*=======================================================================*/

void HEADER_SEI::Read_Seismo ( char *,float  * ,int ,int )
{
	/*
	 long lh;
	 lh=168;
	 INQSGH_C (header,&lh);
	 INQNSA( &nsampl );
	 INQITI( &t0     );
	 INQTST( &dt     );
	 INQAMP( &amp    );
	 INQCOT( &icot   );
	 INQICP( &zer[0][0], &nzer, &pol[0][0], &npol );
	 INQAMC( &amp1, &amp2 );
	if ( fn > st + 3 ) {    nsampl = (long)(fn - st);
	            t0     += (float)st;
	           }
	*/
};

/*=======================================================================*/

void HEADER_SEI::Write_Seismo ( QString &fileOut,float *set,bool )
{


//   char	buf[81];
	// int	i;
//    int tempint,end,sw;
//   long	nelem;
//    float	*data,*dptr,*stop;
	ahhed	ahhead;
	XDR	xdr_out;
	FILE * file_out;

	if ( ( file_out = fopen ( fileOut.toAscii(),"w" ) ) == NULL )
	{
		QMessageBox::critical ( 0,"Error",
		                        QString ( "can't open  file to write'%1'" ).arg ( fileOut ) );
		return;
	}

	xdrstdio_create ( &xdr_out, file_out, XDR_ENCODE );

	strcpy ( ahhead.station.code,st_name );
	strcpy ( ahhead.station.chan,instr );
	ahhead.station.chan[strlen ( instr ) ] = comp[0];
	strcpy ( ahhead.station.stype,"no info" );


	ahhead.station.slat = slat;
	ahhead.station.slon = slong;
	ahhead.station.elev = elevation;

	int i1 = 0;
	ahhead.station.DS = amp2;
	ahhead.station.A0 = amp1;
	ahhead.station.cal[0].pole.r   = ( float ) npol;
	ahhead.station.cal[0].pole.i   = ( float ) 0.0;
	ahhead.station.cal[0].zero.r   = ( float ) nzer;
	ahhead.station.cal[0].zero.i   = ( float ) 0.0;
	for ( i1=0; i1< npol; i1++ )
	{
		ahhead.station.cal[i1+1].pole.r   = pol[i1][0];
		ahhead.station.cal[i1+1].pole.i   = pol[i1][1];

	}
	for ( i1 = npol + 1; i1 < NOCALPTS;i1++ )
	{
		ahhead.station.cal[i1].pole.r   = ( float ) 0.0;
		ahhead.station.cal[i1].pole.i   = ( float ) 0.0;
	}
	for ( i1=0; i1< nzer; i1++ )
	{
		ahhead.station.cal[i1+1].zero.r   = zer[i1][0];
		ahhead.station.cal[i1+1].zero.i   = zer[i1][1];

	}

	for ( i1 = nzer + 1; i1 < NOCALPTS;i1++ )
	{
		ahhead.station.cal[i1].zero.r   = ( float ) 0.0;
		ahhead.station.cal[i1].zero.i   = ( float ) 0.0;
	}
	// }
	ahhead.event.lat = elat;
	ahhead.event.lon = elong;
	ahhead.event.dep = depth;

	ahhead.event.ot.yr = ( short ) year;
	ahhead.event.ot.mo = ( short ) month;
	ahhead.event.ot.day = ( short ) day;
	ahhead.event.ot.hr = ( short ) hour;
	ahhead.event.ot.mn = ( short ) minute;
	ahhead.event.ot.sec = sec;
	strcpy ( ahhead.event.ecomment,"no comment" );


	ahhead.record.type = ( short ) 1;
	ahhead.record.ndata = nsampl;
	ahhead.record.delta = dt;
	float max = set[0];
	float dat = 0.0;
	for ( long i = 0; i < Get_nsampl();i++ )
	{
		dat = ( float ) fabs ( set[i] );
		if ( dat  > max ) max = dat;
	}
	ahhead.record.maxamp = max;

	int ayear;
	int amonth;
	int aday;
//    int doy;
	int ahour;
	int aminute;
	float asecond;
	//---------------------------------------------------------------------
	double start_time =     time_ahead (
	                            ( int )     year,
	                            ( int ) month,
	                            ( int ) day,
	                            ( int ) hour,
	                            ( int ) minute,
	                            sec ) + ( double ) t0;
	//---------------------------------------------------------------------

	time_back ( start_time,&ayear,&amonth,&aday,&ahour,&aminute,&asecond );

	ahhead.record.abstime.yr = ( short ) ayear;
	ahhead.record.abstime.mo = ( short ) amonth;
	ahhead.record.abstime.day = ( short ) aday;
	ahhead.record.abstime.hr = ( short ) ahour;
	ahhead.record.abstime.mn = ( short ) aminute;
	ahhead.record.abstime.sec = ( short ) asecond;
	ahhead.record.rmin = 0.0;
	strcpy ( ahhead.record.rcomment,"null" );
	strcpy ( ahhead.record.log,"null" );


	for ( i1=0; i1<NEXTRAS; ++i1 )
	{
		ahhead.extra[i1] =0.0;
		if ( i1==10 ) ahhead.extra[i1] = shift + tpwave;
	}

	if ( xdr_putrecord ( &ahhead, ( char * ) set, &xdr_out ) != 0 )
	{
		QMessageBox::critical ( 0,"Error",
		                        QString ( "writing ah record" ) );
		return;;
	}
	fclose ( file_out );
}
/*=======================================================================*/
bool  SEISMO::Read_Header_Sei ( const QString  &name ,long ,int indDetrend )
{
//QMessageBox::critical(0,"warning",
//                         QString("Sies file '%1'").arg(name));
	FILE	*file_in;
	if ( ( file_in = fopen ( name.toAscii(),"r" ) ) == NULL )
	{
		QMessageBox::critical ( 0,"Error",
		                        QString ( "can't open '%1'" ).arg ( name ) );
		return false;
	}

	ahhed head;
	head = getahhead ( file_in );
	char* sett = new char [head.record.ndata*sizeof ( float ) ];
//set = new float [head.record.ndata];
	getahdata ( file_in,head,sett );
	set = new float [head.record.ndata];
	memcpy ( set,sett, ( size_t ) ( head.record.ndata*sizeof ( float ) ) );
	delete [] sett;
	long nelem = head.record.ndata;
	float dt =  head.record.delta;
	double t0;
	float Rad = ( float ) ( ( float ) 4.* ( float ) atan ( 1. ) / ( float ) 180. );
	float Deg = ( float ) 180./ ( ( float ) 4.* ( float ) atan ( 1. ) );

	float   az     ;
	float   del    ;
	float   dis    ;
	float   azto   ;
	float   azfr   ;
	float   ela    ;
	float   elon   ;
	float   sla    ;
	float   slon   ;
	float   err_slat;
	float **zzer;
	float **ppol;
	int nzzer;
	int nppol;
	int year    = head.event.ot.yr;
	int month   = head.event.ot.mo;
	int day     = head.event.ot.day;
	int hour    = head.event.ot.hr;
	int minute  = head.event.ot.mn;
	float sec     = head.event.ot.sec;

	Set_year ( head.event.ot.yr );
	Set_month ( head.event.ot.mo );
	Set_day ( head.event.ot.day );
	Set_hour ( head.event.ot.hr );
	Set_minute ( head.event.ot.mn );
	Set_sec ( head.event.ot.sec );

	t0 =     time_ahead (
	             ( int )     head.record.abstime.yr,
	             ( int )     head.record.abstime.mo,
	             ( int )     head.record.abstime.day,
	             ( int )     head.record.abstime.hr ,
	             ( int )     head.record.abstime.mn ,
	             head.record.abstime.sec
	         ) -

	         time_ahead ( ( int ) year, ( int ) month, ( int ) day, ( int ) hour, ( int ) minute, sec );

	Set_t0 ( ( float ) t0 );
	Set_shift ( 0 );
	Set_tpwave ( 0 );
	float pickarr = head.extra[10];
	if ( fabs ( pickarr ) > 1.e-5 ) Set_pickarrival ( head.extra[10] );
	else Set_pickarrival ( ( float ) t0 );
//QMessageBox::critical(0,"warning",
//                       QString("mn-e  '%1' mn-r %2").arg(minute).arg(head.record.abstime.mn));
	Set_elat ( head.event.lat );
	Set_elong ( head.event.lon );
	Set_depth ( head.event.dep );
	ela  = Get_elat();
	elon = Get_elong();

	ela  = ela  * Rad;
	elon = elon * Rad;


	sla  = head.station.slat*Rad;
	slon = head.station.slon*Rad;

	azidmod_ ( &sla,&slon,&ela,&elon,&del,&dis,&azfr,&azto,&err_slat );
	if ( err_slat > 0.001 )
	{
		QMessageBox::information ( 0,"Information",
		                           QString ( "Azimuth calculation: Station (%1) latitude is reduced by  %2 degree." ).arg ( QString ( head.station.code ) ).arg ( err_slat ) );
	}

//azidel_(&sla,&slon,&ela,&elon,&del,&dis,&azfr,&azto);
	Set_dist ( dis );
	Set_aztos ( azto * Deg );
	Set_azfrs ( azfr * Deg );
	Set_takeoffang ( 0 );
	char work[128];
	strcpy ( work,"from AH format (P-inv package). " );

	Set_header ( work );
	QString chanCode = QString ( head.station.chan );
	chanCode = chanCode.trimmed();
	// QString tmp = QString();
	// for (int i = 0; i < 3; ++i) tmp += chanCode[i];
	// chanCode = tmp;
	QString chan = chanCode;
	//chan.remove ( chan.length() - 1,1 );
	Set_instr ( chan.toAscii().data() );
	QString stationCode = QString ( head.station.code );
	stationCode = stationCode.trimmed();
	Set_slat ( head.station.slat ) ;
	Set_st_name ( stationCode.toAscii().data() ) ;
	Set_slong ( head.station.slon ) ;
	Set_elevation ( head.station.elev ) ;
	Set_nsampl ( nelem ) ;
	Set_dt ( dt ) ;

	char ptr1;
//Set_comp   ( head.station.chan + 2   );
	// ptr =  head.station.chan + 2;
	// *ptr = tolower(*ptr);
	ptr1 =  chanCode[chanCode.length() - 1].toAscii();
	Set_comp ( &ptr1 );
	ptr1 = tolower ( ptr1 );
	char *ptr = &ptr1;
	if ( !memcmp ( ptr ,"e",1 ) )
	{
		Set_polan ( ( float ) 90.0 ) ;
		Set_az ( ( float ) 90.0 ) ;
	}

	if ( !memcmp ( ptr ,"n",1 ) )
	{
		Set_polan ( ( float ) 90.0 ) ;
		Set_az ( ( float ) 0.0 ) ;
	}
	if ( !memcmp ( ptr ,"z",1 ) )
	{
		Set_polan ( ( float ) 0.0 ) ;
		Set_az ( ( float ) 0.0 ) ;
	}

	az =  Get_azfrs() - ( float ) 180.0 ;
	if ( !memcmp ( ptr ,"r",1 ) )
	{
		Set_polan ( ( float ) 90.0 ) ;
		Set_az ( ( ( az ) < 0 ) ? ( az + ( float ) 360.0 ) : ( az ) ) ;
	}

	az = Get_azfrs() - ( float ) 90.0 ;
	if ( !memcmp ( ptr ,"t",1 ) )
	{
		Set_polan ( ( float ) 90.0 ) ;
		Set_az ( ( ( az ) < 0 ) ? ( az + ( float ) 360.0 ) : ( az ) ) ;
	}


///                                detrend             ///
	if ( indDetrend == 1 )
	{
		double  flt = ( float ) 0.0;
		for ( long i = 0; i < Get_nsampl();i++ ) flt += set[i];
		flt = flt/Get_nsampl();
		for ( long i = 0; i < Get_nsampl();i++ ) set[i] -= ( float ) flt;
	}
///

//float dat =0.0;
	/*
	float max = set[0];

	      for (long i = 0; i < Get_nsampl();i++)  {
	                dat = (float)fabs (set[i]);
	                if ( dat  > max ) max = dat;
	      }
	      for (long i = 0; i < Get_nsampl();i++) set[i] /= max;
	      */
	Get_max ( 1.0 );
	Set_amp ( 1.0/ head.station.DS ) ;
	Set_DS ( head.station.DS );
	Set_icot ( 1L ) ;
	Set_amp1 ( head.station.A0 ) ;
	Set_amp2 ( head.station.DS ) ;

	nppol  = ( int )       head.station.cal[0].pole.r;
	nzzer  = ( int )       head.station.cal[0].zero.r;

	ppol  = new float *[nppol];
	zzer  = new float *[nzzer];

	for ( long i = 0; i < nppol;i++ )
	{
		ppol[i]  = new float [2];
		ppol[i][0]  = head.station.cal[i+1].pole.r;
		ppol[i][1]  = head.station.cal[i+1].pole.i;
	}
	for ( long i = 0; i < nzzer;i++ )
	{
		zzer[i]  = new float [2];
		zzer[i][0]  = head.station.cal[i+1].zero.r;
		zzer[i][1]  = head.station.cal[i+1].zero.i;
	}

	Set_zer_pol ( nzzer, zzer,nppol,ppol );

	for ( long i = 0; i < nppol;i++ )  delete ppol[i];
	for ( long i = 0; i < nzzer;i++ )  delete zzer[i];
	delete [] zzer;
	delete [] ppol;


	return true;
}
/*=======================================================================*/
void _SET_SEI::RecalculateMaxSeis()
{
	for ( int j = 0;j < I_N_SEI ; j++ )
	{
		SEISMO *ps = Get_S ( j );
		float *set = ps->Get_set();
		long nsampl = ps->Get_nsampl();
		float dat =0.0;
		float max = ( float ) fabs ( set[0] );
		for ( long i = 0; i < nsampl;i++ )
		{
			dat = ( float ) fabs ( set[i] );
			if ( dat  > max ) max = dat;
		}
		for ( long i = 0; i < nsampl;i++ ) set[i] /= max;
		ps->Set_amp ( max * ps->Get_amp( ) ) ;
	}
	Recalc_parametrs();

}
const long Ymin = 1899L;
double time_ahead ( int y, int  m, int d,int h,int mn,float sc )
{
	long year, month, day, hour, minute;
	long i,   year1,  day1;
	int  mm[13] =  {0,31,28,31,30,31,30,31,31,30,31,30,31};
	double time,second;

	year   = ( long ) y   ;
	month  = ( long ) m   ;
	day    = ( long ) d   ;
	hour   = ( long ) h   ;
	minute = ( long ) mn  ;
	second = ( double ) sc;
	year1 = year - Ymin;

	day1 = year1*365 + year1/4;
	if ( year1 > 0 ) day1 += 1;
	for ( i =  1; i < month  ; i++ )   day1 += mm[i];
	if ( ( year1 % 4 == 0 ) && ( month > 2 ) ) day1 += 1;
	day1  +=  day - 1;
	time = ( double ) ( ( day1*24L +  hour )  *60L + minute ) *60.
	       + second;

	return ( time );
}

void time_back ( double time, int *y,int *m, int *d,int *h,int *mn,float *sc )
{
	long    year1,      day1,day2,day3;
	long    year,    month,  day, ttime;
	double secc;
	int  i, mm[13] =  {0,31,28,31,30,31,30,31,31,30,31,30,31};
	ttime = ( long ) ( time/ ( 60. * 60L ) / 24L );
	day2 = day1 = ttime;
	i= 0;
	while ( day2 >= 0L )
		if ( i%4 == 0 ) { day2 -=366L;i++;}
		else  {day2 -=365L;i++;}
	year1 = ( long ) ( i-1 );
	year =year1 + Ymin;


	day2 = year1*365 + year1/4;
	if ( year1 > 0L ) day2 += 1L;

	day1 = day1 - day2;
	day3 = day1;
	i = 1;
	while ( day1 >= 0L ) {day1 -=mm[i]; i++;}
	month = i-1;


	day2 = 0;
	for ( i =  1; i < month  ; i++ )
		day2 += mm[i];
	if ( ( year1 % 4 == 0L ) && ( month > 2 ) ) day2 += 1L;

	day  =  day3 - day2 + 1L;
	secc = time  - ( double )
	       ( ( long ) ( time / ( 60. * 60.* 24. ) ) * ( 60. * 60.* 24. ) );

	*y   = ( int )   year  ;
	*m   = ( int )   month ;
	*d   = ( int )   day   ;
	*h   = ( int ) ( secc / ( 60. * 60. ) );
	*mn  = ( int ) ( secc -  *h  * 60. * 60. ) /60;
	*sc  = ( float ) ( secc - *h  * 60. * 60.  - *mn * 60. );

}

