#include <QtGui>
#include <qwt_plot_printfilter.h>
#include <qwt_plot_marker.h>
#include <qwt_plot_curve.h>
#include <qwt_plot_grid.h>
#include <qwt_picker.h>
#include <qwt_plot_picker.h>
#include <qwt_plot_panner.h>
#include <qwt_legend.h>
#include <qwt_text.h>
#include <math.h>
#include "mainwindow.h"
#include "default.h"
MainWindow *MainWindow::self = 0;
MainWindow::~MainWindow() {
}
MainWindow::MainWindow ( QList<float> &XData,QList<float> &YData,QString psFileName ) {
    self = this;

    Xdata = &XData;
    Ydata = &YData;
    setWindowTitle ( mainWindowCaption() );
    setWindowIcon ( QIcon ( ":/images/icon.png" ) );

    createActions();
    createMenus();
    createToolBars();
    createStatusBar();
    readSettings();

//QMessageBox::critical(0,"",QString("psFileName %1").arg(psFileName));
    plot = new Plot ( this,XData,YData );
    d_picker = new QwtPlotPicker ( QwtPlot::xBottom, QwtPlot::yLeft,
                                   QwtPicker::PointSelection | QwtPicker::DragSelection,
                                   QwtPlotPicker::CrossRubberBand, QwtPicker::AlwaysOn,
                                   plot->canvas() );
    d_picker->setRubberBandPen ( QColor ( Qt::green ) );
    //d_picker->setRubberBand(QwtPicker::CrossRubberBand);
    d_picker->setTrackerPen ( QColor ( Qt::black ) );
    //  d_picker->setResizeMode(QwtPicker::KeepSize);
    d_panner = new QwtPlotPanner ( plot->canvas() );
    d_panner->setMouseButton ( Qt::MidButton );
    setCentralWidget ( plot );
    QFont f = font();
    plot->ChangeFont ( f );
    d_picker->setTrackerFont ( f );
    SavePSFile ( psFileName );
}

void MainWindow::closeEvent ( QCloseEvent *event ) {

    writeSettings();
    event->accept();

}
void MainWindow::about() {
    QString text =  tr("<h2>depth version 1.40.1</h2>"
                       "<p>Copyright &copy; 2008 MITPAN."
                       "<p><b>DbCreator</b> is a program for drawing <b>Residuals</b> as a function of source depth. "
                       "<p>See http://www.mitp.ru/en/soft/mainIndex.html "
                      );
    QMessageBox::about ( this, tr ( "About Application" ),
                         text );
}
void MainWindow::aboutQwt() {
    QString text = tr("<h2>About Qwt</h2>"
                      "<p>Qwt version is 5.1.0 "
                      "<p><b>Qwt</b> is <b>Qt</b> windows for technical applications."
                      "<p><b>Qwt library</b>contains GUI Components and utility classes, "
                      "which are primarily useful for programs with a technical background "
                      "(2D plots, histograms and etc.)."
                      "<p><b>Qwt</b> provides single-source portability across "
                      "<b>Linux</b> , <b>Unix</b> , <b>Windows</b>  and <b>Macintosh</b>."
                      "<p>See http://qwt.sourceforge.net "
                     );
    QMessageBox msgb(QMessageBox::NoIcon, "About Qwt",text);
    msgb.setIconPixmap (QIcon(":/images/qwtwidget.png").pixmap (QSize(60,60)));
    msgb.setWindowIcon(QIcon(":/images/icon.png"));
    msgb.exec();
}

void MainWindow::createActions() {
    exitAct = new QAction ( QIcon ( ":/images/exit.png" ), tr ( "E&xit" ), this );
    exitAct->setShortcut ( tr ( "Ctrl+Q" ) );
    exitAct->setStatusTip ( tr ( "Exit the application" ) );
    connect ( exitAct, SIGNAL ( triggered() ), this, SLOT ( close() ) );


    zoominAct = new QAction ( QIcon ( ":/images/zoom-in.png" ), tr ( "Zoom &In" ),this );
    zoominAct->setShortcut ( tr ( "Ctrl++" ) );
    zoominAct->setStatusTip ( tr ( "Zoom in" ) );
    connect ( zoominAct, SIGNAL ( triggered() ),this, SLOT ( zoomin() ) );



    zoomoutAct = new QAction ( QIcon ( ":/images/zoom-out.png" ), tr ( "Zoom &Out" ),this );
    zoomoutAct->setShortcut ( tr ( "Ctrl+-" ) );
    zoomoutAct->setStatusTip ( tr ( "Zoom out" ) );
    connect ( zoomoutAct, SIGNAL ( triggered() ),this, SLOT ( zoomout() ) );

    printAct = new QAction ( QIcon ( ":/images/print.png" ), tr ( "&Print" ),this );
    printAct->setShortcut ( tr ( "Ctrl+P" ) );
    printAct->setStatusTip ( tr ( "Print depth curve" ) );
    connect ( printAct, SIGNAL ( triggered() ),this, SLOT ( print() ) );

    pdfAct = new QAction ( QIcon ( ":/images/pdf.png" ), tr ( "Save as p&df file" ),this );
    pdfAct->setShortcut ( tr ( "Ctrl+D" ) );
    pdfAct->setStatusTip ( tr ( "Save depth curve as p&df file" ) );
    connect ( pdfAct, SIGNAL ( triggered() ),this, SLOT ( pdf() ) );


    psAct = new QAction ( QIcon ( ":/images/printps.png" ), tr ( "Sav&e as ps file" ),this );
    psAct->setShortcut ( tr ( "Ctrl+E" ) );
    psAct->setStatusTip ( tr ( "Save plots as postscript file" ) );
    connect ( psAct, SIGNAL ( triggered() ),this, SLOT ( ps() ) );

    exportAct = new QAction ( QIcon ( ":/images/fileexport.png" ), tr ( "Save as &text file" ),this );
    exportAct->setShortcut ( tr ( "Ctrl+T" ) );
    exportAct->setStatusTip ( tr ( "Save depth curve as text file" ) );
    connect ( exportAct, SIGNAL ( triggered() ),this, SLOT ( exportData() ) );


    aboutAct = new QAction ( tr ( "&About" ), this );
    aboutAct->setStatusTip ( tr ( "Show the application's About box" ) );
    connect ( aboutAct, SIGNAL ( triggered() ), this, SLOT ( about() ) );

    aboutQwtAct = new QAction ( tr ( "&About Q&wt" ), this );
    aboutQwtAct->setStatusTip ( tr ( "Show the Qwt library's About box" ) );
    connect ( aboutQwtAct, SIGNAL ( triggered() ), this, SLOT ( aboutQwt() ) );

    aboutQtAct = new QAction ( tr ( "About &Qt" ), this );
    aboutQtAct->setStatusTip ( tr ( "Show the Qt library's About box" ) );
    connect ( aboutQtAct, SIGNAL ( triggered() ), qApp, SLOT ( aboutQt() ) );


}

void MainWindow::createMenus() {


    helpMenu = menuBar()->addMenu ( tr ( "&Help" ) );
    helpMenu->addAction ( printAct );
    helpMenu->addAction ( psAct );
    helpMenu->addAction ( pdfAct );
    helpMenu->addAction ( exportAct );
    helpMenu->addSeparator();
    helpMenu->addAction ( zoominAct );
    helpMenu->addAction ( zoomoutAct );
    helpMenu->addAction ( aboutAct );
    helpMenu->addAction ( aboutQwtAct );
    helpMenu->addAction ( aboutQtAct );
    helpMenu->addSeparator();
    helpMenu->addAction ( exitAct );
}

void MainWindow::createToolBars() {
    fileToolBar = addToolBar ( tr ( "File" ) );

    fileToolBar->addAction ( printAct );
    fileToolBar->addAction ( psAct );
    fileToolBar->addAction ( pdfAct );
    fileToolBar->addAction ( exportAct );

    fileToolBar->addSeparator ();

    fileToolBar->addAction ( zoominAct );
    fileToolBar->addAction ( zoomoutAct );
    fileToolBar->addSeparator ();

    fileToolBar->addAction ( exitAct );

}

void MainWindow::createStatusBar() {
    statusBar()->showMessage ( tr ( "Ready" ) );
}
void MainWindow::zoomin() {
    QFont f ( font() );
    f.setPointSize ( f.pointSize() + 1 );
    setFont ( f );
    plot->ChangeFont ( f );

    d_picker->setTrackerFont ( f );
}
void MainWindow::zoomout() {
    QFont f ( font() );
    f.setPointSize ( f.pointSize() - 1 );
    setFont ( f );
    plot->ChangeFont ( f );
    d_picker->setTrackerFont ( f );

}
void MainWindow::readSettings() {
    QSettings settings ( "mitp.ru", "MomTens Draw Depth" );
    QPoint pos = settings.value ( "pos", QPoint ( 100, 100 ) ).toPoint();
    QSize size = settings.value ( "size", QSize ( 400, 600 ) ).toSize();
    resize ( size );
    move ( pos );
    int fontPointSize = settings.value ( "FontPointSize", 10 ).toInt();
    QFont f ( font() );
    f.setPointSize ( fontPointSize );
    setFont ( f );

    saveUserTableDirectory   = settings.value ( "saveUserTableDirectory",userHomePath() ).toString();

    saveUserTableDirectory = QDir::toNativeSeparators ( saveUserTableDirectory );
}

void MainWindow::writeSettings() {
    QSettings settings ( "mitp.ru", "MomTens Draw Depth" );
    settings.setValue ( "pos", pos() );
    settings.setValue ( "size", size() );
    settings.setValue ( "FontPointSize", font().pointSize() );

    settings.setValue ( "saveUserTableDirectory",saveUserTableDirectory );
}

QString MainWindow::strippedName ( const QString &fullFileName ) {
    return QFileInfo ( fullFileName ).fileName();
}
const QString MainWindow::AbsDir ( const QString fileName ) const {

    if ( !fileName.isEmpty() ) return QFileInfo ( fileName ).absoluteDir ().absolutePath ();
    else return QDir::homePath ();



}


void MainWindow::print() {
#if 0
    QPrinter printer;
#else
    QPrinter printer;//(QPrinter::HighResolution);

#endif
    printer.setOutputFormat ( QPrinter::NativeFormat );
    QString docName = plot->title().text();
    if ( !docName.isEmpty() ) {
        docName.replace ( QRegExp ( QString::fromLatin1 ( "\n" ) ), tr ( " -- " ) );
        printer.setDocName ( docName );
    }

    printer.setCreator ( "Bode example" );
    printer.setOrientation ( QPrinter::Portrait );

#if QT_VERSION >= 0x040000
    QPrintDialog dialog ( &printer );
    if ( dialog.exec() ) {
#else
    if ( printer.setup() ) {
#endif
        QwtPlotPrintFilter filter;
        if ( printer.colorMode() == QPrinter::GrayScale ) {
            int options = QwtPlotPrintFilter::PrintAll;
            options &= ~QwtPlotPrintFilter::PrintBackground;
            options |= QwtPlotPrintFilter::PrintFrameWithScales;
            filter.setOptions ( options );
        }

        QPainter printerpainter;
        if ( !printerpainter.begin ( &printer ) )             // paint on printer
            return;
///----------------------------------------------------------------------------
        int pheght = plot->height();
        int pwidth = plot->width();
        float ratio = ( float ) pwidth/ ( float ) pheght;

        int prwidth = printer.width();
        int prheght = printer.height();


        int nheght = prheght;
        int nwidth = prwidth;

        if ( ratio > 1.0 ) {
            //	prwidth = printer.width() *0.8;
            //	prheght = prwidth*1.5;
            nwidth  = nwidth*0.8;
            nheght = nwidth/ratio;
        } else {
            nheght = nheght*0.5;
            nwidth = nheght*ratio;

        }
        int nx = ( printer.width() - nwidth ) /2;
        QRect rect ( nx,prheght*0.05,nwidth,nheght );

///----------------------------------------------------------------------------
        plot->print ( &printerpainter, rect,filter );
    }
}

void MainWindow::ps() {

    QString fileName = QFileDialog::getSaveFileName (
                           this, "Export File Name", userHomePath(),
                           "PostScript Documents (*.ps)" );
    if ( fileName.isEmpty() ) return;
//=============================================================================
    QString locatorStr = ".ps";
    int ind = fileName.length () - locatorStr.length ()-1;
    if ( ind < 0 ) ind =0;
    if ( ( fileName.indexOf ( locatorStr.toLower(),ind ) ) == -1 && ( fileName.indexOf ( locatorStr.toUpper(),ind ) ) == -1 ) {
        fileName.append ( locatorStr );
        if ( QFile::exists ( fileName ) ) {
            int ret = QMessageBox::warning ( this, mainWindowCaption(),
                                             QString ( "File %1 already exists.\n Do you want to overwrite it?" )
                                             .arg ( QDir::convertSeparators ( fileName ) ),
                                             QMessageBox::Yes | QMessageBox::Default,
                                             QMessageBox::No | QMessageBox::Escape );
            if ( ret == QMessageBox::No )
                return ;
        }
    }
//=============================================================================
    QPrinter printer;
    QwtPlotPrintFilter filter;

    if ( printer.colorMode() == QPrinter::GrayScale ) {
        int options = QwtPlotPrintFilter::PrintAll;
        options &= ~QwtPlotPrintFilter::PrintBackground;
        options |= QwtPlotPrintFilter::PrintFrameWithScales;
        filter.setOptions ( options );
    }

    printer.setOutputFormat ( QPrinter::PostScriptFormat );
    printer.setOrientation ( QPrinter::Portrait );
    printer.setOutputFileName ( fileName );

    printer.setCreator ( "Bode example" );
    QPainter printerpainter;
    if ( !printerpainter.begin ( &printer ) )             // paint on printer
        return;
///----------------------------------------------------------------------------
    int pheght = plot->height();
    int pwidth = plot->width();
    float ratio = ( float ) pwidth/ ( float ) pheght;

    int prwidth = printer.width();
    int prheght = printer.height();


    int nheght = prheght;
    int nwidth = prwidth;

    if ( ratio > 1.0 ) {
        //	prwidth = printer.width() *0.8;
        //	prheght = prwidth*1.5;
        nwidth  = nwidth*0.8;
        nheght = nwidth/ratio;
    } else {
        nheght = nheght*0.5;
        nwidth = nheght*ratio;

    }
    int nx = ( printer.width() - nwidth ) /2;
    QRect rect ( nx,prheght*0.05,nwidth,nheght );

///----------------------------------------------------------------------------

    plot->print ( &printerpainter, rect,filter );

    QFont f = font();
    plot->ChangeFont ( f );

}
void MainWindow::SavePSFile ( QString fileName ) {
    QPrinter printer;
    QwtPlotPrintFilter filter;
    if ( printer.colorMode() == QPrinter::GrayScale ) {
        int options = QwtPlotPrintFilter::PrintAll;
        options &= ~QwtPlotPrintFilter::PrintBackground;
        options |= QwtPlotPrintFilter::PrintFrameWithScales;
        filter.setOptions ( options );
    }
    printer.setOutputFormat ( QPrinter::PostScriptFormat );
    printer.setOrientation ( QPrinter::Portrait );
    printer.setOutputFileName ( fileName );

    printer.setCreator ( "Bode example" );
    QPainter printerpainter;
///----------------------------------------------------------------------------
    QRect r ( 0,0,printer.width(),printer.height() );
    int nheght = r.height() *0.5;
    int nwidth = ( int ) ( ( float ) nheght/1.5 );
    int nx = ( r.width() - nwidth ) /2;
    QRect rect ( nx,r.y() + r.height() *0.05,nwidth,nheght );
///----------------------------------------------------------------------------
//QMessageBox::about(0,"",QString("ratio %1").arg((float)rect.height()/(float)rect.width()));
    if ( !printerpainter.begin ( &printer ) )             // paint on printer
        return;
    plot->print ( &printerpainter, rect,filter );
    QFont f = font();
    plot->ChangeFont ( f );

}
void MainWindow::pdf() {


    QString fileName = QFileDialog::getSaveFileName (
                           this, "Export File Name", userHomePath(),
                           "PDF Documents (*.pdf)" );
    if ( fileName.isEmpty() ) return;
//=============================================================================
    QString locatorStr = ".pdf";
    int ind = fileName.length () - locatorStr.length ()-1;
    if ( ind < 0 ) ind =0;
    if ( ( fileName.indexOf ( locatorStr.toLower(),ind ) ) == -1 && ( fileName.indexOf ( locatorStr.toUpper(),ind ) ) == -1 ) {
        fileName.append ( locatorStr );
        if ( QFile::exists ( fileName ) ) {
            int ret = QMessageBox::warning ( this, mainWindowCaption(),
                                             QString ( "File %1 already exists.\n Do you want to overwrite it?" )
                                             .arg ( QDir::convertSeparators ( fileName ) ),
                                             QMessageBox::Yes | QMessageBox::Default,
                                             QMessageBox::No | QMessageBox::Escape );
            if ( ret == QMessageBox::No )
                return ;
        }
    }
//=============================================================================

    QPrinter printer;
    QwtPlotPrintFilter filter;
    if ( printer.colorMode() == QPrinter::GrayScale ) {
        int options = QwtPlotPrintFilter::PrintAll;
        options &= ~QwtPlotPrintFilter::PrintBackground;
        options |= QwtPlotPrintFilter::PrintFrameWithScales;
        filter.setOptions ( options );
    }
    printer.setOutputFormat ( QPrinter::PdfFormat );
    printer.setOrientation ( QPrinter::Portrait );
    printer.setOutputFileName ( fileName );

    printer.setCreator ( "Bode example" );
    QPainter printerpainter;
    if ( !printerpainter.begin ( &printer ) )             // paint on printer
        return;
///----------------------------------------------------------------------------
    int pheght = plot->height();
    int pwidth = plot->width();
    float ratio = ( float ) pwidth/ ( float ) pheght;

    int prwidth = printer.width();
    int prheght = printer.height();


    int nheght = prheght;
    int nwidth = prwidth;

    if ( ratio > 1.0 ) {
        //	prwidth = printer.width() *0.8;
        //	prheght = prwidth*1.5;
        nwidth  = nwidth*0.8;
        nheght = nwidth/ratio;
    } else {
        nheght = nheght*0.5;
        nwidth = nheght*ratio;

    }
    int nx = ( printer.width() - nwidth ) /2;
    QRect rect ( nx,prheght*0.05,nwidth,nheght );

///----------------------------------------------------------------------------

    plot->print ( &printerpainter, rect,filter );
    QFont f = font();
    plot->ChangeFont ( f );
}

void MainWindow::exportData() {
    QString fileName =
        QFileDialog::getSaveFileName ( this,"Save the curves as text file under a new name",
                                       saveUserTableDirectory ,
                                       saveCurveFilters() );
// +"polarity.sup",
    if ( fileName.isEmpty() ) return;
    int ind = fileName.length () - saveCurveFileFilterLocator().length ()-1;
    if ( ind < 0 ) ind =0;
    if ( ( fileName.indexOf ( saveCurveFileFilterLocator(),ind ) ) == -1 &&
            ( fileName.indexOf ( saveCurveFileFilterLocator().toUpper(),ind ) ) == -1 ) {
        fileName.append ( saveCurveFileFilterLocator() );
        if ( QFile::exists ( fileName ) ) {
            int ret = QMessageBox::warning ( this, mainWindowCaption(),
                                             QString ( "File %1 already exists.\n Do you want to overwrite it?" )
                                             .arg ( QDir::toNativeSeparators ( fileName ) ),
                                             QMessageBox::Yes | QMessageBox::Default,
                                             QMessageBox::No | QMessageBox::Escape );
            if ( ret == QMessageBox::No )
                return;
        }
    }

    if ( !fileName.isEmpty() ) {
        saveUserTableDirectory = QFileInfo ( fileName ).absolutePath ();

        QFile file ( fileName );
        if ( !file.open ( QIODevice::WriteOnly | QIODevice::Text ) ) {
            QMessageBox::warning ( this,"",QString ( "Can't open file for writing\n %1" ).arg ( fileName ) );
            return;
        }
        QTextStream out ( &file );

        for ( int i = 0; i < Xdata->size(); ++i ) {
            out << QString ( "%1 %2\n" ).arg ( Xdata->at ( i ) ).arg ( Ydata->at ( i ) );
        }


        file.close();
    }
}
///===================================================================================
Plot::Plot ( QWidget *parent,QList<float> &X,QList<float> &Y ) : QwtPlot ( parent ) {

    double cur1Y[X.size() ];
    double cur1X[X.size() ];
    float maxY = Y.at ( 0 );
    for ( int i = 0; i < X.size(); ++i ) {
        float y = Y.at ( i );
        if ( y > maxY ) maxY = y;
        cur1Y[i] = y;
        cur1X[i] = X.at ( i );
    }

    enableAxis ( QwtPlot::yRight );
    enableAxis ( QwtPlot::yLeft );
    enableAxis ( QwtPlot::xBottom );
    setAxisAutoScale ( QwtPlot::xBottom );
    setAxisAutoScale ( QwtPlot::yLeft );
    setAxisAutoScale ( QwtPlot::yRight );

    setAutoReplot ( true );
    QPalette pl = MainWindow::self->palette();
    QBrush br = pl.window();
    setCanvasBackground ( br.color() );

    setMargin ( 10 );

    QwtPlotGrid *grid = new QwtPlotGrid;
    grid->enableXMin ( true );
    grid->enableYMin ( true );
    grid->setMajPen ( QPen ( Qt::darkGray, 0, Qt::DotLine ) );
    grid->setMinPen ( QPen ( Qt::lightGray, 0 , Qt::DotLine ) );
    grid->attach ( this );

    //setAxisMaxMinor(QwtPlot::xBottom,0);


    //  setAxisTitle(QwtPlot::yLeft, "Normalized misfit");

    cSin = new QwtPlotCurve ( "First branch" );
    cSin->setRenderHint ( QwtPlotItem::RenderAntialiased );
    QPen PenFirst ( Qt::blue );
    PenFirst.setWidth ( 3 );
    cSin->setPen ( PenFirst );
    cSin->setData ( cur1X,cur1Y,X.size() );

    setAxisScale ( QwtPlot::yRight,cSin->minYValue(),cSin->maxYValue() );

    setAxisScale ( QwtPlot::yLeft,cSin->minYValue(),cSin->maxYValue() );
//    cSin->setYAxis(QwtPlot::yLeft);
//    cSin->setYAxis(QwtPlot::yRight);
    cSin->attach ( this );
    //replot();
}

void   Plot::ChangeFont ( QFont &f ) {

//QMessageBox::warning ( this, mainWindowCaption(),"ChangeFont");
///--------------------------------------------------------------------------
    QFont AxisFont = f;
    AxisFont.setPointSize ( AxisFont.pointSize() + 2 );
    AxisFont.setBold ( true );

    QwtText qtext;
    qtext.setFont ( AxisFont );
    qtext.setColor ( Qt::darkBlue );
    qtext.setText ( "Residual as function of source depth" );
    setTitle ( qtext );
    qtext.setText ( "Depth,km" );
    setAxisTitle ( QwtPlot::xBottom, qtext );
    setAxisFont ( QwtPlot::xBottom,f );
    setAxisFont ( QwtPlot::yLeft,f );
    setAxisFont ( QwtPlot::yRight,f );

///----------------------------------------------------------------------------

    replot();
}

void Plot::print ( QPainter *painter,const QRect &plotRect,
                   const QwtPlotPrintFilter &pfilter ) const {

    QwtPlot::print ( painter,plotRect, pfilter );

    QPen oldPen1 = painter->pen();
    painter->setPen ( QPen ( Qt::black ) );
    painter->drawRect ( plotRect );
    painter->setPen ( oldPen1 );
    /*
        QPen oldPen1 = p->pen();
        p->setPen ( QPen ( Qt::black ) );
    //    p->drawRect(r);
        p->setPen ( oldPen1 );
    */
}







