#include <QtGui>
#include "mainwindow.h"
#include <QMessageBox>
#include <stdlib.h>
#include <qwt_math.h>
#include <qwt_plot_canvas.h>
#include <qframe.h>
#include <qtooltip.h>
#include "mainwindow.h"
#include "bode_plot.h"
#include "default.h"
#include "bode.h"
#include "dataset.h"


void delBodePlotAndNullify(BodePlot* &pBodePlot) {
    delete  pBodePlot;
    pBodePlot = 0;
}
void delFrameAndNullify(QFrame* &pFrame) {
    delete  pFrame;
    pFrame = 0;
}
//----------------------------------------------------------------------------------------------------------------------------------------------
GraphWin::~GraphWin() {

    for_each(vBodePlot.begin(),vBodePlot.end(),delBodePlotAndNullify);
    vBodePlot.erase(remove(vBodePlot.begin(),vBodePlot.end(),
                           static_cast<BodePlot* >(0)),
                    vBodePlot.end());

    for_each(vFrame.begin(),vFrame.end(),delFrameAndNullify);
    vFrame.erase(remove(vFrame.begin(),vFrame.end(),
                        static_cast<QFrame* >(0)),
                 vFrame.end());

}
GraphWin::GraphWin(QWidget *p , CDataInfo* dataInfo,itPlot it1, itPlot it2,int normalization) :
        QWidget(p),dataInfo(dataInfo),indNormalization(normalization) {
//  vector<CCurve* > CVector = project->GetCurvec();
    dataInfo->GetMinMaxValues();

    numRow = 3;
    numCol  = 3;
    numCurve = 0;

    for ( vector<CDataSet* >::iterator itt = (*it1); itt != (*it2); ++itt) numCurve++;
    MainWindowLayout = new QGridLayout( this);
    groupBox = new QGroupBox( this );


    // groupBox->layout()->setSpacing ( 6 );

    //  groupBox->layout()->setContentsMargins(4,4,4,4 );

    groupBoxLayout = new QGridLayout( groupBox);

    groupBoxLayout->setAlignment( Qt::AlignTop );
    // Window Hider
    textLabel1 = new QLabel( groupBox);
    textLabel1->setAlignment(Qt::AlignHCenter);
    textLabel1->setTextFormat( Qt::RichText );

    header = "Rayleigh amplitude spectra as function of period";
    QString  indLR =  (*(*it1))->GetSortPrefix();

//   QMessageBox::warning(0,"",QString("header '%1'indLR '%2' indLR '%3'").arg(header).arg(indLR).arg(indLR));
    if (indLR.toLower()[0] == 'l') {
        if (header.indexOf("Rayleigh") != -1)     header.replace( "Rayleigh", "Love");
        if (header.indexOf("rayleigh")  != -1)    header.replace( "rayleigh", "love");
    }
    if (indLR.toLower()[0] == 'r') {
        if (header.indexOf("Love")  != -1)    header.replace( "Love", "Rayleigh");
        if (header.indexOf("love")  != -1)    header.replace( "love", "rayleigh");
    }
    textLabel1->setText(header );



    // groupBoxLayout->addWidget( textLabel1, 0, 0, 0, numCol -1 );

    textLabel2 = new QLabel( groupBox);
    textLabel2->setAlignment(Qt::AlignHCenter);
    textLabel2->setTextFormat(Qt::RichText);

    vector<CDataSet* >::iterator itF = (*it1);
    QString maximumY =" ";
    if (indNormalization == 0) {
        maximumY.sprintf("Max Y: %le", dataInfo->GetMaxYCommon());
    }
    if (indNormalization == 1) {
        if ((*itF)->GetSortPrefix().toLower()[0] == 'l' ) maximumY.sprintf("Max Y: %le", dataInfo->GetMaxYLove());
        if ((*itF)->GetSortPrefix().toLower()[0] == 'r' ) maximumY.sprintf("Max Y: %le", dataInfo->GetMaxYRayleigh());


    }

    footer = maximumY;
        QString tmp = "";

        if (MainWindow::self->projectVersion == momentTensorProfileName())
            tmp = " observed: <font color=red>red</font>; synthetic: <font color=blue>blue</font> ";
        if (MainWindow::self->projectVersion == secondMomentProfileName())
            tmp = " observed: <font color=red>red</font>; synthetic: <font color=blue>blue</font> - zero-moments, <font color=green>green</font> - 2nd moments";

    
     footer += tmp;

   // footer = "<font color=blue> ----- </font>  <b> First curve; </b> <font color=red> ----- </font>  <b>        Second curve. </b> ";
 textLabel2->setTextFormat(Qt::RichText);
    textLabel2->setText( footer  );


    int i = 0;

    for ( vector<CDataSet* >::iterator itta = (*it1); itta != (*it2); ++itta) {

        BodePlot *d_plot1 = new BodePlot(groupBox,(*itta),dataInfo,QString(""),indNormalization);
        vBodePlot.push_back(d_plot1);
        i++;
    }


    vector<BodePlot* >::iterator  it = vBodePlot.begin();
    int j=0;
    int k=0;
    for (j=0;j<numRow;j++) {
        for (k=0;k<numCol;k++) {
            groupBoxLayout->addWidget((*it), j+1,k);
            it++;
            if (it == vBodePlot.end()) break;
        }
        if (it == vBodePlot.end()) break;
    }
//--------------------------------------------------------------------------------------------------------------------------------------------------

    vector<CDataSet* >::iterator dFirst = (*it1);
    if (  numCurve < numCol)
        for (int m =k+1 ; m <numCol; m++ ) {
            QString name = QString("PlotHide %1").arg(m+1);
            BodePlotHide *d_plot2 = new BodePlotHide(groupBox,(*dFirst),dataInfo,indNormalization);
            groupBoxLayout->addWidget(d_plot2 ,   j+1,m);
        }
    for (int m = j +1 ; m < numRow; m++ ) {
        QString name = QString("PlotHide %1").arg(m+1);
        BodePlotHide *d_plot2 = new BodePlotHide(groupBox,(*dFirst),dataInfo,indNormalization);
        groupBoxLayout->addWidget(d_plot2,   m + 2,0);


    }

//-------------------------------------------------------------------------------------------------------------------------------------------------------------
    // groupBoxLayout->addWidget( textLabel2, numRow+2, numRow+2);
    MainWindowLayout->addWidget( textLabel1, 0, 0);
    MainWindowLayout->addWidget( groupBox, 1, 0 );
    MainWindowLayout->addWidget( textLabel2, 2, 0 );

    // show();
///???????????????
    // clearWState( Qt::WState_Polished );



}
void GraphWin::ChangeFont(QFont f) {
    QFont labelFont = f;
    labelFont.setPointSize(labelFont.pointSize() + 1 );
    textLabel1->setFont(labelFont);
    textLabel1->setText(textLabel1->text());

    textLabel2->setFont(labelFont);
    textLabel2->setText(textLabel2->text());
    /*
        //AxisFont.setBold(true);
        QwtText qtext("Depth,km");
        qtext.setFont (AxisFont);
        qtext.setColor (Qt::darkBlue);
        //setAxisTitle(QwtPlot::xBottom, qtext);
        qtext.setText(titleStr);
        setTitle(qtext);
    */
}

