#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qmessagebox.h>
#include "dataset.h"
#include "default.h"
#include <float.h>
#include <math.h>

struct data_Comp : public binary_function < CDataSet * const & , CDataSet *  const &, bool> {
    bool operator()(CDataSet *  const & x, CDataSet*  const & y) {
        if (x->GetSortPrefix() > y->GetSortPrefix() )
            return false;
        else return true;
    }
};



//--------------------------------------- CDataInfo  --------------------------------------------------------------------------------------------------------------------------------------
void delDataSetAndNullify(CDataSet* &pCDataSet) {
    delete  pCDataSet;
    pCDataSet = 0;
}
CDataInfo::~CDataInfo() {
    for_each(DataSet.begin(),DataSet.end(),delDataSetAndNullify);
    DataSet.erase(remove(DataSet.begin(),DataSet.end(),
                         static_cast<CDataSet*>(0)),
                  DataSet.end());
}
CDataInfo::CDataInfo(const QString &nameDir,const QString &projectOpenName):
        PlaceDirectory(nameDir),projectOpenName(projectOpenName) {

    QDir dirTemp(nameDir);
    QStringList list = dirTemp.entryList();

    if (projectOpenName == momentTensorProfileName())  {

        QStringList fields = momentTensorLocatorFileName().split(' ');
        QStringList::iterator itf = fields.begin();
        QStringList::iterator its = fields.begin();
        its++;
        int nfirstMt = 0;
        int nsecondMt = 0;
        set<QString > listFiles1;
        set<QString > listFiles2;
        bool isProtocol = false;
        for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {

            QString fn =( *it );
            QString fnLower =  fn.toLower ();
            if (fnLower.contains( (*itf)) ) {
                listFiles1.insert( fn );
                nfirstMt++;
            }
            if (fnLower.contains( (*its)) ) {
                listFiles2.insert( fn );
                nsecondMt++;
            }
            if (fnLower.contains("protocol.txt") ) {
                isProtocol = true;
            }

        }

        QString Protocol = "";
        if (isProtocol) {

            QFile wfd( nameDir + QDir::convertSeparators("/")  + QString("protocol.txt"));
            if ( wfd.open( QIODevice::ReadOnly ) ) {
                QTextStream stream( &wfd );
                while ( !stream.atEnd() ) Protocol += stream.readLine () + "\n";
                wfd.close();
            } else {
                QMessageBox::warning(0,mainWindowCaption(),QString("Get data for plot::> Can't open flle for reading\n%1'").arg("protocol.txt"));

            }

        }
//    if (!Protocol.isEmpty ()) qWarning("%s",Protocol.ascii());
        set<QString >::iterator it1= listFiles1.begin();
        set<QString >::iterator it2 = listFiles2.begin();
        while (  it1!= listFiles1.end() ) {
            QStringList lfile;
            QString az,azorbit,staname;

            QString namePrefix = (*it2);
            namePrefix = namePrefix.left(namePrefix.indexOf('_') );

            lfile.append( nameDir + QDir::convertSeparators("/")  +   (*it1));
            lfile.append( nameDir + QDir::convertSeparators("/")  +   (*it2));
            CDataSet *dataSet = 0;
            if (!Protocol.isEmpty ())   {
                GetHeaderItems(namePrefix,Protocol,az,azorbit,staname);
                dataSet = new CDataSet(lfile,az,azorbit,staname);
            } else   dataSet = new CDataSet(lfile);
            DataSet.push_back(dataSet);
            lfile.clear();
            it1++;
            it2++;


        }


        if (!Protocol.isEmpty ())  std::sort( DataSet.begin(),DataSet.end(),data_Comp());




    }
    if (projectOpenName == secondMomentProfileName())  {
//QMessageBox::critical(0,"","CDataInfo::CDataInfo 0");

        QStringList fields = secondMomentLocatorFileName().split(' ', QString::SkipEmptyParts);
        QStringList::iterator itf = fields.begin();
        QStringList::iterator its = fields.begin();
        its++;
        QStringList::iterator itu = fields.begin();
        itu++;
        itu++;
        int nfirstMt = 0;
        int nsecondMt = 0;
        int ntherdMt = 0;
        set<QString > listFiles1;
        set<QString > listFiles2;
        set<QString > listFiles3;
        bool isProtocol = false;
        for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {

            QString fn =( *it );
            QString fnLower =  fn.toLower ();
            if (fnLower.contains( (*itf)) ) {
                listFiles1.insert( fn );
                nfirstMt++;
            }
            if (fnLower.contains( (*its)) ) {
                listFiles2.insert( fn );
                nsecondMt++;
            }
            if (fnLower.contains( (*itu)) ) {
                listFiles3.insert( fn );
                ntherdMt++;
            }
            if (fnLower.contains( "protocol.txt") ) {
                isProtocol = true;
            }

        }
        QString Protocol = "";
        if (isProtocol) {
            QFile wfd( nameDir + QDir::convertSeparators("/")  + QString("protocol.txt"));
            if ( wfd.open( QIODevice::ReadOnly ) ) {
                QTextStream stream( &wfd );
                while ( !stream.atEnd() ) Protocol += stream.readLine();
                wfd.close();
            } else {
                QMessageBox::warning(0,mainWindowCaption(),QString("Get data for plot::> Can't open flle for reading.\n%1'").arg("protocol.txt"));

            }

        }

//    if (!Protocol.isEmpty ()) qWarning("%s",Protocol.ascii());
        set<QString >::iterator it1= listFiles1.begin();
        set<QString >::iterator it2 = listFiles2.begin();
        set<QString >::iterator it3 = listFiles3.begin();
        while (  it1 != listFiles1.end() ) {
            QStringList lfile;
            QString az,azorbit,staname;
            QString namePrefix = (*it1);
            namePrefix = namePrefix.left(namePrefix.
                                         indexOf('.') );
            lfile.append( nameDir + QDir::convertSeparators("/")  + (*it1) );
            lfile.append(nameDir + QDir::convertSeparators("/")  + (*it2) );
            lfile.append(nameDir + QDir::convertSeparators("/")  + (*it3) );

            CDataSet *dataSet = 0;
            if (!Protocol.isEmpty ())   {
                GetHeaderItems(namePrefix,Protocol,az,azorbit,staname);
                dataSet = new CDataSet(lfile,az,azorbit,staname);
            } else   {
                az = (*it1).right( (*it1).length() - (*it1).
                                   indexOf('.') -1 );
                dataSet = new CDataSet(lfile,az);

            }
            DataSet.push_back(dataSet);
            lfile.clear();
            it1++;
            it2++;
            it3++;


        }

        // if (!Protocol.isEmpty ())
        std::sort( DataSet.begin(),DataSet.end(),data_Comp());


    }
}
void CDataInfo::GetHeaderItems(QString namePrefix,QString &protocol,QString &az,QString &azorbit,QString &staname) {

    int index1 = protocol.indexOf(namePrefix);
    int index2 =  protocol.indexOf("Station_Name",index1);
    int index3 =  protocol.indexOf("=",index2);
    int index4 =  protocol.indexOf("Seis_Name",index3);
    QString StationName =  protocol.mid(index3 + 1,index4 - index3 -1);
    StationName.remove(' ');
    int index5 = protocol.indexOf("Aztost",index4);
    int index6 = protocol.indexOf("=",index5);
    int index7 = protocol.indexOf("\n",index6);
    QString azimus =  protocol.mid(index6 + 1,index7 - index6 -1);
    azimus.remove(' ');
    QString orbit = StationName.right( StationName.length() - StationName.indexOf(':') -1);
    int orb = orbit.toInt();
    float azflt = azimus.toFloat();
    float azNew = 0.0;
    if ( (orb/2)*2 == orb) {
        azNew = azflt+ 180.0;
        if (azNew > 360.0) azNew -= 360.0;
    } else azNew = azflt;

    char buf[128];
    ::sprintf(buf, "%7.2f",azflt);
    az = QString::fromAscii(buf);
    ::sprintf(buf, "%7.2f",azNew);
    azorbit = QString::fromAscii(buf);
    az.remove (' ');
    azorbit.remove (' ');
//QMessageBox::about(0, "",QString("az %1 azorbit %2" ).arg(azimus).arg(azorbit));
    staname = StationName;

}
void CDataInfo::PrintDataInfo() {
    /*
        qWarning("Print info data to plot:");
        qWarning("projectOpenName: '%s'", projectOpenName.toAscii());
        qWarning("PlaceDirectory: '%s'", PlaceDirectory.ascii());
        qWarning("Data Det:> ");
        DataSetVectorIterator it;
        int j = 0;
        for ( it = DataSet.begin(); it !=DataSet.end(); ++it ) {
    	qWarning(" Plot: %2d  Az: '%s'   AzOrbit '%s'\n        StaName '%s' SortPrefix '%s'",j+1,(*it)->GetAz().ascii(),
    		 (*it)->GetAzOrbit().ascii(), (*it)->GetNameStation().ascii(),(*it)->GetSortPrefix().ascii());
    	j++;
    	vector<CDoubleData* > dvector =  (*it)->GetDoubleData();
    	int i = 0;
    	for ( vector<CDoubleData* >::iterator itt = dvector.begin(); itt != dvector.end(); ++itt){
    	    qWarning("       curve:%d : '%s'  ",i+1,(*itt)->GetFileName().ascii());
    	    i++;
    	}


        }
    */
}
void CDataInfo::GetMinMaxValues() {

    MinXLove =  MinXRayleigh = MinXCommon =  MinYLove =  MinYRayleigh = MinYCommon = DBL_MAX;
    MaxXLove = MaxXRayleigh = MaxXCommon = MaxYLove =  MaxYRayleigh = MaxYCommon = -DBL_MAX;

    for ( DataSetVectorIterator it = DataSet.begin(); it !=DataSet.end(); ++it ) {
        vector<CDoubleData* > dvector =  (*it)->GetDoubleData();
        for ( vector<CDoubleData* >::iterator itt = dvector.begin(); itt != dvector.end(); ++itt) {
            QString justfileName =  (*itt)->GetFileName();
            justfileName =  justfileName.right( justfileName.length() - justfileName.lastIndexOf(QDir::convertSeparators("/") ) -1 );

            if ( justfileName.toLower()[0] == 'l' ) {
                if ((*itt)->GetMinX() < MinXLove ) MinXLove= (*itt)->GetMinX() ;
                if ((*itt)->GetMinY() < MinYLove ) MinYLove= (*itt)->GetMinY() ;
                if ((*itt)->GetMaxX() > MaxXLove ) MaxXLove= (*itt)->GetMaxX() ;
                if ((*itt)->GetMaxY() > MaxYLove ) MaxYLove= (*itt)->GetMaxY() ;
            }
            if (justfileName.toLower()[0] == 'r' ) {
                if ((*itt)->GetMinX() < MinXRayleigh) MinXRayleigh = (*itt)->GetMinX() ;
                if ((*itt)->GetMinY() < MinYRayleigh)  MinYRayleigh = (*itt)->GetMinY() ;
                if ((*itt)->GetMaxX() > MaxXRayleigh) MaxXRayleigh = (*itt)->GetMaxX() ;
                if ((*itt)->GetMaxY() > MaxYRayleigh) MaxYRayleigh  = (*itt)->GetMaxY() ;
            }
        } // itt
    } // it
    if  (MinXLove <  MinXRayleigh) MinXCommon = MinXLove;
    else MinXCommon = MinXRayleigh;
    if  (MinYLove <  MinYRayleigh) MinYCommon = MinYLove;
    else MinYCommon = MinYRayleigh;
    if  (MaxXLove >  MaxXRayleigh) MaxXCommon = MaxXLove;
    else MaxXCommon = MaxXRayleigh;
    if  (MaxYLove >  MaxYRayleigh) MaxYCommon = MaxYLove;
    else MaxYCommon = MaxYRayleigh;

}
//--------------------------------------- CDataSet --------------------------------------------------------------------------------------------------------------------------------------

void delDoubleDataAndNullify(CDoubleData* &pCDoubleData) {
    delete  pCDoubleData;
    pCDoubleData = 0;
}
CDataSet::~CDataSet() {

    for_each(DoubleData.begin(),DoubleData.end(),delDoubleDataAndNullify);
    DoubleData.erase(remove(DoubleData.begin(),DoubleData.end(),
                            static_cast<CDoubleData*>(0)),
                     DoubleData.end());
}

CDataSet::CDataSet(QStringList &flist,QString az,QString azorbit,QString staname):
        Az(az),AzOrbit(azorbit),NameStation(staname) {
    QString pref =  (*(flist.begin()));


    pref  = pref .right(pref .length() - pref .lastIndexOf(QDir::convertSeparators("/") ) -1 );
    pref  = pref.left(pref .indexOf('_') );

    if (Az.isEmpty () && AzOrbit.isEmpty () && NameStation.isEmpty () ) SortPrefix =   pref ;
    else {

        if (! Az.isEmpty () && AzOrbit.isEmpty () && NameStation.isEmpty ()) {
            float azFlt = Az.toFloat();
            QString az;
            char buf[128];
            ::sprintf(buf, "% 4.0f",azFlt);
            az = QString::fromAscii(buf);
            if (fabs(az.toFloat()) < 100) az = " " + az;
            if (fabs(az.toFloat()) < 10) az = " " + az;
            SortPrefix = pref[0] +"   az:"+ az;
        } else {
            QString tmp = AzOrbit;
            if (fabs(AzOrbit.toFloat()) < 100) tmp = " " + tmp;
            if (fabs(AzOrbit.toFloat()) < 10) tmp = " " + tmp;
            SortPrefix = pref[0] + tmp + NameStation;
        }
    }

   // QMessageBox::about(0,"",SortPrefix);
//   qWarning("XXX   Start '%s'",SortPrefix.ascii());
    for ( QStringList::Iterator it = flist.begin(); it != flist.end(); ++it ) {
////	qWarning("XXX   '%s'",(*it).ascii());
        CDoubleData *ddata = new CDoubleData((*it));
        DoubleData.push_back( ddata);
    }
//        qWarning("XXX  Finish");

}
//--------------------------------------- CDoubleData --------------------------------------------------------------------------------------------------------------------------------------
CDoubleData::~CDoubleData() {
    if ( X != 0) {
        delete [] X;
        X = 0;
    }
    if ( Y != 0) {
        delete [] Y;
        Y = 0;
    }
}
CDoubleData::CDoubleData(const QString &fileName): FileName(fileName),X(0),Y(0) {

    QStringList lines;
    QFile fd(fileName);
    Size = 0;
    if (!fd.open(QFile::ReadOnly | QFile::Text)) {
        QMessageBox::warning(0,mainWindowCaption(),
                             QString("Cannot read file %1:\n%2.")
                             .arg(fileName)
                             .arg(fd.errorString()));
        return;
    }


    QTextStream stream( &fd );
    QString line;

    do {
        line = stream.readLine();
        QStringList lst = line.split(' ', QString::SkipEmptyParts);
        if (lst.size() < 2) break;
        lines << line;
    } while (!line.isNull());

    fd.close();
    int sizeList = (int)lines.size ();
    X = new double[sizeList];
    Y = new double[sizeList];
    int i = 0;
    for (int j = 0; j < lines.size(); ++j) {
        //   sscanf(lines.at(j).toAscii(),"%le%le",&X[i],&Y[i]);

        QStringList lst = lines.at(j).split(' ', QString::SkipEmptyParts);
        X[j] = lst.at(0).toFloat();
        Y[j] = lst.at(1).toFloat();

    }

    Size = sizeList;

    if (Size > 0 ) {
        MinX = MinY = DBL_MAX;
        MaxX = MaxY = -DBL_MAX;
        for (int i = 0;i < Size;++i) {
            if (X[i] <MinX ) MinX = X[i];
            if (Y[i] <MinY ) MinY = Y[i];
            if (X[i] >MaxX ) MaxX = X[i];
            if (Y[i] >MaxY ) MaxY = Y[i];
        }

    }

}
