#ifndef  PLOT_DATA_SET_H
#define PLOT_DATA_SET_H
#include <qstring.h> 
#include <qstringlist.h>
#include <vector>
#include <set>
#include <functional>
#include <algorithm>

using namespace std;
class CCurve;
class CDoubleData
{
public:
    CDoubleData(){}
    CDoubleData(const QString &fileName);
    ~CDoubleData();    
    QString &GetFileName(){return FileName;}
    double* &GetX(){return X;}
    double* &GetY(){return Y;}
    double &GetMinX(){return MinX;}
    double &GetMaxX(){return MaxX;}    
    double &GetMinY(){return MinY;}    
    double &GetMaxY(){return MaxY;}
    int         &GetSize() { return Size;}
private:
    QString FileName;
    int Size;
    double* X;
    double* Y;
    double MinX;
    double MaxX;    
    double MinY;    
    double MaxY;
};
class CDataSet
{
    typedef vector<CDoubleData* > DataVector;
    typedef vector<CDoubleData* >::iterator DataVectorIterator;    
public:
    CDataSet(){}
    CDataSet(QStringList &listfilenames,QString az ="",QString AzOrbit = "",QString NameStation = "");
    ~CDataSet();
    QString &GetSortPrefix(){ return SortPrefix;}
	const char* GetSortPrefixStr(){ return SortPrefix.toAscii().data();}
   QString &GetAz(){ return Az;}
   QString &GetAzOrbit(){ return AzOrbit;} 
   QString &GetNameStation(){ return NameStation;}     
   DataVector &GetDoubleData(){ return DoubleData;}
private:
    QString SortPrefix;
    QString Az;
    QString AzOrbit;  
    QString NameStation;        
    DataVector DoubleData;    
};
class CDataInfo
{
    typedef vector<CDataSet* > DataSetVector;
    typedef vector<CDataSet* >::iterator DataSetVectorIterator;    
public:
    CDataInfo(){}
    CDataInfo(const QString &nameDir,const QString &projectOpenName);
    ~CDataInfo(); 
    void PrintDataInfo();
    void GetHeaderItems(QString namePrefix,QString &protocol,QString&az,QString&azorbit,QString&staname);
    
    void GetMinMaxValues();
    
    QString &GetPlaceDirectory(){ return PlaceDirectory;}
    double &GetMinXLove(){ return MinXLove;}
    double &GetMinXRayleigh(){ return MinXRayleigh;}
    double &GetMinXCommon(){ return MinXCommon;}    
    double &GetMaxXLove(){ return MaxXLove;}
    double &GetMaxXRayleigh(){ return MaxXRayleigh;}
    double &GetMaxXCommon(){ return MaxXCommon;}    
    
    double &GetMinYLove(){ return MinYLove;}
    double &GetMinYRayleigh(){ return MinYRayleigh;}
    double &GetMinYCommon(){ return MinYCommon;}     
    double &GetMaxYLove(){ return MaxYLove;}
    double &GetMaxYRayleigh(){ return MaxYRayleigh;}
    double &GetMaxYCommon(){ return MaxYCommon;}    
    
    DataSetVector &GetDataSet(){ return DataSet;}     

private:
    QString PlaceDirectory;
    QString projectOpenName;
    double MinXLove;
    double MinXRayleigh;
    double MinXCommon;     
    double MaxXLove;
    double MaxXRayleigh;
    double MaxXCommon;    
    
    double MinYLove;
    double MinYRayleigh;
    double MinYCommon;     
    double MaxYLove;
    double MaxYRayleigh;
    double MaxYCommon;    
    
    DataSetVector DataSet;    
};
#endif
