#include <QtGui>
#include <QMessageBox>
#include "mainwindow.h"
#include  <math.h>
#include <default.h>
QString M0MTENS_ROOT_PATH;
QString M0MTENS_SEARCH_HELP_PATH;
int main(int argc, char *argv[]) {
    Q_INIT_RESOURCE(obscal);

    QApplication app(argc, argv);
    if ( argc < 4) {
        QMessageBox::critical(0,mainWindowCaption(),"usage: canv \"data directory\" first/second \"PostScript file name\"");
        return 1;
    }

    QString dirName = QString(argv[1]);
    QString version = QString(argv[2]);

    if (!(QFileInfo (dirName).isDir())) {
        QMessageBox::critical(0,mainWindowCaption(),
                              QString("Directory '%1' does not exist").arg(dirName),
                              QMessageBox::Close);
        return 1;
    }

    QDir tempDir(dirName);
    tempDir.setFilter(QDir::Files | QDir::NoSymLinks);
    QFileInfoList infolist = tempDir.entryInfoList();

    if (!(version.contains("first") || version.contains("second") )) {
        QMessageBox::critical(0,mainWindowCaption(),
                              QString("Second argument must be first or second.").arg(dirName),
                              QMessageBox::Close);
        return 1;
    }
    if (version.contains("first")) {
        bool obsExist = false;
        bool calExist = false;
        for ( int j = 0; j < infolist.size(); ++j ) {
            QFileInfo  infoCurTemp = infolist.at ( j );
            QString fileName = infoCurTemp.filePath().toLower();
            if ( fileName.contains (QDir::toNativeSeparators ( "/." )) ) continue;
            if ( fileName.contains ( QDir::toNativeSeparators ("/.." )) ) continue;
            if ( fileName.contains ( "_obs") ) obsExist = true;
            if ( fileName.contains ( "_cal") ) calExist = true;

        }

        if (!obsExist && !calExist) {
            QMessageBox::critical(0,mainWindowCaption(),
                                  QString("Directory '%1' does not contain drawing marerials").arg(dirName),
                                  QMessageBox::Close);
            return 1;

        }
        if (!obsExist) {
            QMessageBox::critical(0,mainWindowCaption(),
                                  QString("Directory '%1' does not contain observed data for drawing").arg(dirName),
                                  QMessageBox::Close);
            return 1;

        }

        if (!calExist) {
            QMessageBox::critical(0,mainWindowCaption(),
                                  QString("Directory '%1' does not contain synthetic data for drawing").arg(dirName),
                                  QMessageBox::Close);
            return 1;

        }
    }

if (version.contains("second")) {
QStringList lst = secondMomentLocatorFileName().split(' ', QString::SkipEmptyParts);
        bool m1oa_tExist = false;
        bool inca_tExist = false;
        bool fica_tExist = false;
        for ( int j = 0; j < infolist.size(); ++j ) {
            QFileInfo  infoCurTemp = infolist.at ( j );
            QString fileName = infoCurTemp.filePath().toLower();
            if ( fileName.contains (QDir::toNativeSeparators ( "/." )) ) continue;
            if ( fileName.contains ( QDir::toNativeSeparators ("/.." )) ) continue;
            if ( fileName.contains ( lst.at(0) ) ) m1oa_tExist = true;
            if ( fileName.contains ( lst.at(1) ) ) inca_tExist = true;
            if ( fileName.contains ( lst.at(2) ) ) fica_tExist = true;

        }

        if (!m1oa_tExist && !inca_tExist  && !fica_tExist) {
            QMessageBox::critical(0,mainWindowCaption(),
                                  QString("Directory '%1' does not contain drawing marerials").arg(dirName),
                                  QMessageBox::Close);
            return 1;

        }
        if (!m1oa_tExist) {
            QMessageBox::critical(0,mainWindowCaption(),
                                  QString("Directory '%1' does not contain m1oa_t data for drawing").arg(dirName),
                                  QMessageBox::Close);
            return 1;

        }

        if (!inca_tExist) {
            QMessageBox::critical(0,mainWindowCaption(),
                                  QString("Directory '%1' does not contain inca_t data for drawing").arg(dirName),
                                  QMessageBox::Close);
            return 1;

        }

        if (!fica_tExist) {
            QMessageBox::critical(0,mainWindowCaption(),
                                  QString("Directory '%1' does not contain fica_t data for drawing").arg(dirName),
                                  QMessageBox::Close);
            return 1;

        }
    }
    app.setStyle(QStyleFactory::create("plastique"));
    QApplication::setPalette(QApplication::style()->standardPalette());
    QString absoluteInstallEcpPath = app.applicationDirPath();
    int pos = absoluteInstallEcpPath.lastIndexOf("/bin");
    absoluteInstallEcpPath.remove ( pos, 4 );
    M0MTENS_ROOT_PATH = absoluteInstallEcpPath;
#if defined(Q_WS_WIN)
    M0MTENS_SEARCH_HELP_PATH = "file:///" + M0MTENS_ROOT_PATH + "/doc";
#endif
#if defined(Q_WS_X11)
    M0MTENS_SEARCH_HELP_PATH = "file://" + M0MTENS_ROOT_PATH + "/doc";
#endif
#if defined(Q_WS_MAC)
    M0MTENS_SEARCH_HELP_PATH = "file://" + M0MTENS_ROOT_PATH + "/doc";
#endif


    MainWindow mainWin(dirName,version,QString(argv[3]));
    mainWin.show();
    return app.exec();
}
