#include <QtGui>
#include <QPrinter>
#include <qwt_plot_printfilter.h>
#include "mainwindow.h"
#include "default.h"
#include "dataset.h"
#include "bode.h"
#include "bode_plot.h"
MainWindow *MainWindow::self = 0;
void delBodePlotAndNullify2(BodePlot* &pBodePlot) {
    delete  pBodePlot;
    pBodePlot = 0;
}
MainWindow::~MainWindow() {
    if ( graphWin !=0) delete graphWin;
}
MainWindow::MainWindow(QString &dirName,QString &version,QString psFileName ) : version(version), dataDirName(dirName),psFileName(psFileName)  {

    self = this;
    graphWin = 0;
    setWindowTitle ( mainWindowCaption());
    setWindowIcon (QIcon(":/images/icon.png"));

    createActions();
    createMenus();
    createToolBars();
    createStatusBar();
    readSettings();

    if (version.contains("first"))  projectVersion = momentTensorProfileName();
    if (version.contains("second")) projectVersion = secondMomentProfileName();
    loadProjectDirectory(dataDirName);
    show();
    SavePSFile(psFileName);
}

void MainWindow::closeEvent(QCloseEvent *event) {

    writeSettings();
    event->accept();

}
void MainWindow::about() {
    QString text =  tr("<h2>depth version 1.40.1</h2>"
                       "<p>Copyright &copy; 2008 MITPAN."
                       "<p><b>DbCreator</b> is a program for drawing <b>observed and synthetic</b> surface wave amplitude spectra calculated for the best values of variable parameters. "
                       "<p>See http://www.mitp.ru/en/soft/mainIndex.html "
                      );
    QMessageBox::about ( this, tr ( "About Application" ),
                         text );
}

void MainWindow::aboutQwt() {
    QString text = tr("<h2>About Qwt</h2>"
                      "<p>Qwt version is 5.1.0 "
                      "<p><b>Qwt</b> is <b>Qt</b> windows for technical applications."
                      "<p><b>Qwt library</b>contains GUI Components and utility classes, "
                      "which are primarily useful for programs with a technical background "
                      "(2D plots, histograms and etc.)."
                      "<p><b>Qwt</b> provides single-source portability across "
                      "<b>Linux</b> , <b>Unix</b> , <b>Windows</b>  and <b>Macintosh</b>."
                      "<p>See http://qwt.sourceforge.net "
                     );
    QMessageBox msgb(QMessageBox::NoIcon, "About Qwt",text);
    msgb.setIconPixmap (QIcon(":/images/qwtwidget.png").pixmap (QSize(60,60)));
    msgb.setWindowIcon(QIcon(":/images/icon.png"));
    msgb.exec();
}
void MainWindow::createActions() {
    exitAct = new QAction(QIcon(":/images/exit.png"), tr("E&xit"), this);
    exitAct->setShortcut(tr("Ctrl+Q"));
    exitAct->setStatusTip(tr("Exit the application"));
    connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));


    forwardAct = new QAction(QIcon(":/images/forward.png"), tr("&Forward"), this);
    forwardAct->setShortcut(tr("Ctrl+F"));
    forwardAct->setStatusTip(tr("Show next page "));
    connect(forwardAct, SIGNAL(triggered()), this, SLOT(forward()));


    backAct = new QAction(QIcon(":/images/back.png"), tr("&Back"), this);
    backAct->setShortcut(tr("Ctrl+B"));
    backAct->setStatusTip(tr("Show back page "));
    connect(backAct, SIGNAL(triggered()), this, SLOT(back()));

    zoominAct = new QAction(QIcon(":/images/zoom-in.png"), tr("Zoom &In"),this);
    zoominAct->setShortcut(tr("Ctrl++"));
    zoominAct->setStatusTip(tr("Zoom in"));
    connect(zoominAct, SIGNAL(triggered()),this, SLOT(zoomin()));



    zoomoutAct = new QAction(QIcon(":/images/zoom-out.png"), tr("Zoom &Out"),this);
    zoomoutAct->setShortcut(tr("Ctrl+-"));
    zoomoutAct->setStatusTip(tr("Zoom out"));
    connect(zoomoutAct, SIGNAL(triggered()),this, SLOT(zoomout()));

    printAct = new QAction(QIcon(":/images/print.png"), tr("&Print"),this);
    printAct->setShortcut(tr("Ctrl+P"));
    printAct->setStatusTip(tr("Print depth curve"));
    connect(printAct, SIGNAL(triggered()),this, SLOT(print()));

    pdfAct = new QAction(QIcon(":/images/pdf.png"), tr("Save as p&df file"),this);
    pdfAct->setShortcut(tr("Ctrl+D"));
    pdfAct->setStatusTip(tr("Save plots as p&df file"));
    connect(pdfAct, SIGNAL(triggered()),this, SLOT(pdf()));


    psAct = new QAction(QIcon(":/images/printps.png"), tr("Sav&e as ps file"),this);
    psAct->setShortcut(tr("Ctrl+E"));
    psAct->setStatusTip(tr("Save plots as postscript file"));
    connect(psAct, SIGNAL(triggered()),this, SLOT(ps()));

    exportAct = new QAction(QIcon(":/images/fileexport.png"), tr("Save as &text file"),this);
    exportAct->setShortcut(tr("Ctrl+T"));
    exportAct->setStatusTip(tr("Save depth curve as text file"));
    connect(exportAct, SIGNAL(triggered()),this, SLOT(exportData()));


    aboutAct = new QAction(tr("&About"), this);
    aboutAct->setStatusTip(tr("Show the application's About box"));
    connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

    aboutQwtAct = new QAction ( tr ( "&About Q&wt" ), this );
    aboutQwtAct->setStatusTip ( tr ( "Show the Qwt library's About box" ) );
    connect ( aboutQwtAct, SIGNAL ( triggered() ), this, SLOT ( aboutQwt() ) );

    aboutQtAct = new QAction(tr("About &Qt"), this);
    aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
    connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));


}

void MainWindow::createMenus() {


    helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(printAct);
    helpMenu->addAction(psAct);
    helpMenu->addAction(pdfAct);
    //helpMenu->addAction(exportAct);
    helpMenu->addSeparator();
    helpMenu->addAction(zoominAct);
    helpMenu->addAction(zoomoutAct);
    helpMenu->addAction(aboutAct);
    helpMenu->addAction ( aboutQwtAct );
    helpMenu->addAction(aboutQtAct);
    helpMenu->addSeparator();
    helpMenu->addAction(exitAct);
}

void MainWindow::createToolBars() {
    fileToolBar = addToolBar(tr("File"));

    fileToolBar->addAction(printAct);
    fileToolBar->addAction(psAct);
    fileToolBar->addAction(pdfAct);
    //fileToolBar->addAction(exportAct);

    fileToolBar->addSeparator ();

    fileToolBar->addAction(zoominAct);
    fileToolBar->addAction(zoomoutAct);
    fileToolBar->addSeparator ();

    fileToolBar->addAction(backAct);
    fileToolBar->addAction(forwardAct);
    fileToolBar->addSeparator ();

    normaCombo = new QComboBox(fileToolBar);
    QLineEdit* lnedit = new QLineEdit(normaCombo);
    lnedit->setReadOnly(TRUE);
    normaCombo->setLineEdit(lnedit);
    normaCombo->addItem  ("Common");
    normaCombo->addItem ("Group");
    normaCombo->addItem ("Own");
    normaCombo->setToolTip("Normalization type" );
    normaCombo->setAutoCompletion( TRUE );

    connect( normaCombo, SIGNAL(activated ( int  ) ), this, SLOT( setNormalization(int ) ) );
    fileToolBar->addWidget(normaCombo);
    fileToolBar->addSeparator ();

    fileToolBar->addAction(exitAct);

}

void MainWindow::createStatusBar() {
    statusBar()->showMessage(tr("Ready"));
}
void MainWindow::zoomin() {
    QFont f( font() );
    f.setPointSize( f.pointSize() + 1 );
    setFont( f );
    if (graphWin != 0)SetFontToGraph();
}
void MainWindow::zoomout() {
    QFont f(font() );
    f.setPointSize( f.pointSize() - 1 );
    setFont( f );
    if (graphWin != 0)SetFontToGraph();
}
void MainWindow::readSettings() {
    QSettings settings("mitp.ru", "MomTens Draw ObsCal");
    QPoint pos = settings.value("pos", QPoint(100, 100)).toPoint();
    QSize size = settings.value("size", QSize(400, 600)).toSize();
    resize(size);
    move(pos);
    int fontPointSize = settings.value("FontPointSize", 10).toInt();
    QFont f(font());
    f.setPointSize(fontPointSize );
    setFont( f );
}

void MainWindow::writeSettings() {
    QSettings settings("mitp.ru", "MomTens Draw ObsCal");
    settings.setValue("pos", pos());
    settings.setValue("size", size());
    settings.setValue("FontPointSize", font().pointSize());
}

QString MainWindow::strippedName(const QString &fullFileName) {
    return QFileInfo(fullFileName).fileName();
}
const QString MainWindow::AbsDir(const QString fileName) const {

    if (!fileName.isEmpty()) return QFileInfo (fileName).absoluteDir ().absolutePath ();
    else return QDir::homePath ();

}

///==============================================================================

void MainWindow::PrintPrinterPainter(QPrinter &printer,QwtPlotPrintFilter &filter) {
    QPainter printerpainter;
    QFontMetrics fm = printerpainter.fontMetrics();
//        QPaintDeviceMetrics metrics( &printer ); // need width/height
    // of printer surfa
    int heightPlot = printer.height();
    int widthPlot  = printer.width();
    int height =printer.height();
    int width = printer.width();
//QMessageBox::critical(0, "",QString("height %1 width %2").arg(height).arg(width));
    vector<BodePlot* >::iterator  itPlotWin = graphWin->vBodePlot.begin();
    int graphWidth = (*itPlotWin)->width();
    int graphHeight = (*itPlotWin)->height();

    //qWarning ("XXX   PlotWinWidth %d   PlotWinHeight %d   PlotWidth  %d    PlotHeight  %d ",graphWidth,graphHeight,width,height);
    if ( !printerpainter.begin( &printer ))               // paint on printer
        return;

    vector<BodePlot* > vPrintPlot;
    for ( int nPage = 0; nPage  < numberPage; ++nPage) {
        itPlot it1 = viter.begin();
        itPlot it2 = viter.begin();
        it2++;
        for (int ind = 0; ind< nPage;ind++) {
            it1++;
            it2++;
        }
        int i = 0;
        for ( vector<CDataSet* >::iterator itta = (*it1); itta != (*it2); ++itta) {
            QString name = QString("Plot %1").arg(i+1);
            BodePlot *d_plot1 = new BodePlot(this,(*itta),dataInfo,name,indNormalization,true);
            QFont f = printerpainter.font();
            d_plot1->ChangeFont(f);
            vPrintPlot.push_back(d_plot1);
            i++;
        }


        //////////////////////////////////////////////////////////////////////  !!!!!!!!!!!!!!!!!!!!!!!!      ////////////////////////////
        // Draw Page boudaries
        //  printerpainter.drawRect(0,0,widthPlot,heightPlot);
        float commonMargin = (float)0.95;
        height = (int)( commonMargin*printer.height());



        QFont oldFont = printerpainter.font();
        QPen oldPen = printerpainter. pen();
// ----------------------------------- print header ------------------------------------------------------------------ //
        QString header = graphWin->header;
        QString  indLR =  (*(*it1))->GetSortPrefix();


        if (indLR.toLower()[0] == 'l') {
            if (header.indexOf("Rayleigh") != -1)     header.replace( "Rayleigh", "Love");
            if (header.indexOf("rayleigh")  != -1)    header.replace( "rayleigh", "love");
        }
        if (indLR.toLower()[0] == 'r') {
            if (header.indexOf("Love")  != -1)    header.replace( "Love", "Rayleigh");
            if (header.indexOf("love")  != -1)    header.replace( "love", "rayleigh");
        }

        QFont f = printerpainter.font();
        f.setPointSize( f.pointSize() + 4 );
        printerpainter.setFont(f);


        QRect headerRect =printerpainter.boundingRect(0, 0,widthPlot,heightPlot,Qt::AlignHCenter,header);
        headerRect.moveTo(0,headerRect.height());
        //  printerpainter.drawText(headerRect.x(),headerRect.y()+headerRect.height(),header);

        printerpainter.drawText(0,headerRect.y()+headerRect.height(),widthPlot,headerRect.height(),Qt::AlignHCenter,header);
        //printerpainter.drawRect(headerRect);
// ----------------------------------- print header ------------------------------------------------------------------ //

// ----------------------------------- print  footet ------------------------------------------------------------------ //
        QString footer ;
//-------------------------------------------------------------------------------------------------------------------------------------------------------
        vector<CDataSet* >::iterator itF =  *it1;
        QString maximumY ="";
        if (indNormalization == 0) {
            maximumY.sprintf("Max Y: %le", dataInfo->GetMaxYCommon());
        }
        if (indNormalization == 1) {

            if ((*itF)->GetSortPrefix().toLower()[0] == 'l' ) maximumY.sprintf("Max Y: %le", dataInfo->GetMaxYLove());
            if ((*itF)->GetSortPrefix().toLower()[0] == 'r' ) maximumY.sprintf("Max Y: %le", dataInfo->GetMaxYRayleigh());


        }

        footer = maximumY;
        QString tmp = "";

        if (projectVersion == momentTensorProfileName())
            tmp = " black - observed; gray - synthetic. ";
        if (projectVersion == secondMomentProfileName())
            tmp = " black - observed; dark gray - synthetic zero-moments; light gray - synthetic 2nd moments";
        footer += tmp;
//------------------------------------------------------------------------------------------------
        QRect footerRect =printerpainter.boundingRect(0, 0,width,height,Qt::AlignHCenter,footer);
        footerRect.moveTo(0,height);
        //  printerpainter.drawText(footerRect.x(),footerRect.y()+footerRect.height(),footer);
        printerpainter.drawText(0,footerRect.y()+footerRect.height(),widthPlot,footerRect.height(),Qt::AlignHCenter,footer);



        //printerpainter.drawRect(footerRect);
// ----------------------------------- print  footet ------------------------------------------------------------------ //
        printerpainter.setFont(oldFont );
        printerpainter.setPen(oldPen);



        height = height - 3*headerRect.height() - footerRect.height();
        QRect rectMain(0, 3*headerRect.height(),width,height );
        // Draw rect for Draw
        //printerpainter.drawRect(rectMain);

        int drawWigth = rectMain.width();
        int drawHeight = rectMain.height();

        int oldH = graphHeight;

        if ( numRow >1) {
            if ( (graphHeight*numRow )> drawHeight  ) {
                graphHeight =  (int)( (double)drawHeight / numRow);
                graphWidth =(int) ( graphWidth * ( (double)graphHeight / oldH) );
            }
        }  else {
            if (graphHeight > drawHeight  ) {
                graphHeight =  drawHeight;
                graphWidth = (int) (graphWidth * ( (double)graphHeight / oldH) );
            }
        }

        int oldW = graphWidth;

        if ( numCol >1) {
            if ( (graphWidth*numCol )> drawWigth  ) {
                graphWidth =  (int)( (double)drawWigth / numCol);
                graphHeight = (int) (graphHeight * (   (double)graphWidth / oldW) );
            }
        }  else {
            if (graphWidth > drawWigth  ) {
                graphWidth =  drawWigth;
                graphHeight =  (int) (graphHeight * (  (double) graphWidth / oldW) );
            }
        }


        int moving = (int)((double)(drawWigth - graphWidth*numCol ) /2.);
        if (moving > 0)
            rectMain.moveTo(moving, 0);
        int  deltayW = graphWidth;
        int  deltayH = graphHeight;

        //qWarning(" deltayW = %d deltayH = %d",deltayW,deltayH);

        QRect rect =  rectMain;

        rect.setWidth(deltayW);
        rect.setHeight(deltayH);
//QMessageBox::critical(0, "",QString("height %1 width %2").arg(deltayH).arg(deltayW));
        //    Draw set of graphs
        vector<BodePlot* >::iterator  it = vPrintPlot.begin();
        int j=0;
        int k=0;
        for (j=0;j<numRow;j++) {
            for (k=0;k<numCol;k++) {
                rect =  rectMain;
                rect.setWidth(deltayW);
                rect.setHeight(deltayH);
                rect.moveTo(k*deltayW,j*deltayH);

                QRect r;
                int newX = (int)((widthPlot - deltayW*numCol)/2);
                r.setX(rect.x() + newX +(int)(rect.width()*0.01) );
                r.setY(rect.y() + 2*headerRect.bottom ()+(int)(rect.height()*0.01));
                r.setWidth((int)(rect.width()*0.98));
                r.setHeight((int)(rect.height()*0.98));

                //    printerpainter.drawRect(r);

                (*it)->print(&printerpainter, r,filter);

                it++;
                if (it == vPrintPlot.end()) break;
            }
            if (it == vPrintPlot.end()) break;
        }


        for_each(vPrintPlot.begin(),vPrintPlot.end(),delBodePlotAndNullify2);
        vPrintPlot.erase(remove(vPrintPlot.begin(),vPrintPlot.end(),
                                static_cast<BodePlot* >(0)),
                         vPrintPlot.end());
        if (nPage  < numberPage - 1)
            printer.newPage();
        else {
            printerpainter.end();
            break;
        }
    }
}
///==============================================================================
void MainWindow::SavePSFile(QString fileName){
    QPrinter printer;
    printer.setOutputFormat(QPrinter::PostScriptFormat);
    printer.setOrientation(QPrinter::Landscape);
    printer.setOutputFileName(fileName);
    printer.setCreator("Bode example");
    QwtPlotPrintFilter filter;
    if ( printer.colorMode() == QPrinter::GrayScale ) {
        int options = QwtPlotPrintFilter::PrintAll;
        options &= ~QwtPlotPrintFilter::PrintBackground;
        options |= QwtPlotPrintFilter::PrintFrameWithScales;
        filter.setOptions(options);
    }

    PrintPrinterPainter(printer,filter);

}
///==============================================================================
void MainWindow::ps() {
    QString fileName = QFileDialog::getSaveFileName(
                           this, "Export File Name", userHomePath(),
                           "PostScript Documents (*.ps)");
    if ( fileName.isEmpty() ) return;
//=============================================================================
    QString locatorStr = ".ps";
    int ind = fileName.length () - locatorStr.length ()-1;
    if (ind < 0) ind =0;
    if ( ( fileName.indexOf(locatorStr.toLower(),ind)) == -1 && ( fileName.indexOf(locatorStr.toUpper(),ind)) == -1) {
        fileName.append(locatorStr);
        if (QFile::exists(fileName)) {
            int ret = QMessageBox::warning(this, mainWindowCaption(),
                                           QString("File %1 already exists.\n Do you want to overwrite it?")
                                           .arg(QDir::convertSeparators(fileName)),
                                           QMessageBox::Yes | QMessageBox::Default,
                                           QMessageBox::No | QMessageBox::Escape);
            if (ret == QMessageBox::No)
                return ;
        }
    }

    QPrinter printer;
    printer.setOutputFormat(QPrinter::PostScriptFormat);
    printer.setOrientation(QPrinter::Landscape);
    printer.setOutputFileName(fileName);
    printer.setCreator("Bode example");
    QwtPlotPrintFilter filter;
    if ( printer.colorMode() == QPrinter::GrayScale ) {
        int options = QwtPlotPrintFilter::PrintAll;
        options &= ~QwtPlotPrintFilter::PrintBackground;
        options |= QwtPlotPrintFilter::PrintFrameWithScales;
        filter.setOptions(options);
    }

    PrintPrinterPainter(printer,filter);

}

void MainWindow::pdf() {

    QString fileName = QFileDialog::getSaveFileName(
                           this, "Export File Name", userHomePath(),
                           "PDF Documents (*.pdf)");
    if ( fileName.isEmpty() ) return;
//=============================================================================
    QString locatorStr = ".pdf";
    int ind = fileName.length () - locatorStr.length ()-1;
    if (ind < 0) ind =0;
    if ( ( fileName.indexOf(locatorStr.toLower(),ind)) == -1 && ( fileName.indexOf(locatorStr.toUpper(),ind)) == -1) {
        fileName.append(locatorStr);
        if (QFile::exists(fileName)) {
            int ret = QMessageBox::warning(this, mainWindowCaption(),
                                           QString("File %1 already exists.\n Do you want to overwrite it?")
                                           .arg(QDir::convertSeparators(fileName)),
                                           QMessageBox::Yes | QMessageBox::Default,
                                           QMessageBox::No | QMessageBox::Escape);
            if (ret == QMessageBox::No)
                return ;
        }
    }
//=============================================================================

    QPrinter printer;
    printer.setOutputFormat(QPrinter::PdfFormat);
    printer.setOrientation(QPrinter::Landscape);
    printer.setOutputFileName(fileName);
    printer.setCreator("Bode example");
    QwtPlotPrintFilter filter;
    if ( printer.colorMode() == QPrinter::GrayScale ) {
        int options = QwtPlotPrintFilter::PrintAll;
        options &= ~QwtPlotPrintFilter::PrintBackground;
        options |= QwtPlotPrintFilter::PrintFrameWithScales;
        filter.setOptions(options);
    }
    PrintPrinterPainter(printer,filter);



}
void MainWindow::print() {
#if 0
    QPrinter printer;
#else
    QPrinter printer;//(QPrinter::HighResolution);

#endif
    printer.setOutputFormat (QPrinter::NativeFormat);
    QString docName = "";//d_plot->title().text();
    if ( !docName.isEmpty() ) {
        docName.replace (QRegExp (QString::fromLatin1 ("\n")), tr (" -- "));
        printer.setDocName (docName);
    }

    printer.setCreator("Bode example");
    printer.setOrientation(QPrinter::Landscape);

#if QT_VERSION >= 0x040000
    QPrintDialog dialog(&printer);
    if ( dialog.exec() ) {
#else
    if (printer.setup()) {
#endif

        QwtPlotPrintFilter filter;

        if ( printer.colorMode() == QPrinter::GrayScale ) {
            int options = QwtPlotPrintFilter::PrintAll;
            options &= ~QwtPlotPrintFilter::PrintBackground;
            options |= QwtPlotPrintFilter::PrintFrameWithScales;
            filter.setOptions(options);
        }
        PrintPrinterPainter(printer,filter);
    }




}

void MainWindow::exportData() {

}

void MainWindow::SetFontToGraph() {
    QFont f = font();
    vector<BodePlot* >bplot =  graphWin->vBodePlot;
    vector<BodePlot* >::iterator  it = graphWin->vBodePlot.begin();

    for ( it = bplot.begin(); it != bplot.end(); ++it ) ( *it )->ChangeFont(f);
    graphWin->ChangeFont(f);
}
void MainWindow::setNormalization(int norm ) {

//QMessageBox::critical(0, "",normaCombo->itemText (index ));
    indNormalization = norm;
    itPlot it1 = viter.begin();
    itPlot it2 = viter.begin();
    it2++;
    // indResize = 1;
    for (int i = 0; i < currentPage;i++) {
        it1++;
        it2++;
    }
    SetGraphWidget ( it1, it2);
    SetFontToGraph();

}

void MainWindow::forward() {

    if (currentPage + 1 > numberPage -1)   return;
    if (currentPage + 1 > numberPage -2)   forwardAct->setEnabled(FALSE);
    currentPage++;
    backAct->setEnabled(TRUE);


    itPlot it1 = viter.begin();
    itPlot it2 = viter.begin();
    it2++;
//   indResize = 1;
    for (int i = 0; i < currentPage;i++) {
        it1++;
        it2++;
    }

    SetGraphWidget ( it1, it2);
    SetFontToGraph();
}
void MainWindow::back() {
    if (currentPage - 1 <  0 )     return;
    if (currentPage - 1 <  1 )  backAct->setEnabled(FALSE);
    currentPage--;
    forwardAct->setEnabled(TRUE);
    itPlot it1 = viter.begin();
    itPlot it2 = viter.begin();
    it2++;

//indResize = 1;
    for (int i = 0; i < currentPage;i++) {
        it1++;
        it2++;
    }

    SetGraphWidget ( it1, it2);
    SetFontToGraph();

}
void MainWindow::loadProjectDirectory(const QString &dirName) {
    if (projectVersion == momentTensorProfileName())
        dataInfo = new CDataInfo(dirName,momentTensorProfileName());

    if (projectVersion == secondMomentProfileName())
        dataInfo = new CDataInfo(dirName,secondMomentProfileName());
    configure();
//	project->PrintProject();
//dataInfo->PrintDataInfo();
}
void MainWindow::configure() {

    indNormalization = 0;
    currentPage = 0;
    normaCombo->setCurrentIndex(0);
    graphWin = 0;

    backAct->setEnabled(FALSE);
    dvector = dataInfo->GetDataSet();
    numRow = 3;
    numCol  = 3;
    numCurve = dataInfo->GetDataSet().size();
    int numLove = 0;
    int numRayleigh = 0;

    for ( vector<CDataSet* >::iterator itt = dvector.begin(); itt != dvector.end(); ++itt) {

        if ( (*itt)->GetSortPrefix().toLower()[0] == 'l' ) numLove++;
        if ( (*itt)->GetSortPrefix().toLower()[0] == 'r' ) numRayleigh++;
    }

    if (numLove > 0) {
        div_t q = div(numLove, numRow*numCol);
        numberPageLove = q.quot;
        numberPageLoveRem = q.rem;
    } else   numberPageLove =  numberPageLoveRem = 0;

    if (numRayleigh > 0) {
        div_t q = div(numRayleigh, numRow*numCol);
        numberPageRayleigh = q.quot;
        numberPageRayleighRem = q.rem;
    } else numberPageRayleigh = numberPageRayleighRem = 0;

    numberPage = numberPageLove+ numberPageRayleigh ;
    if (numberPageLoveRem > 0) numberPage++;
    if (numberPageRayleighRem > 0) numberPage++;



//    vector< vector<CDataSet* >::iterator > viter;

    vector<CDataSet* >::iterator itt1 = dvector.begin();
    viter.push_back(itt1 );

    if (numLove > 0) {
        for (int i2 = 0; i2 < numberPageLove; i2++) {
            for (int j = 0; j < numRow*numCol;j++)  itt1++;
            viter.push_back(itt1 );
        }

        for (int i1 = 0; i1 < numberPageLoveRem; i1++)  itt1++;
        viter.push_back(itt1 );
    }

    if (numRayleigh > 0) {
        for (int i3 = 0; i3 < numberPageRayleigh; i3++) {
            for (int j1 = 0; j1 < numRow*numCol;j1++)  itt1++;
            viter.push_back(itt1 );
        }

        for (int i4 = 0; i4 < numberPageRayleighRem; i4++)  itt1++;
        viter.push_back(itt1 );
    }

    itPlot it1 = viter.begin();
    itPlot it2 = viter.begin();
    it2++;
    SetGraphWidget ( it1, it2);
    SetFontToGraph();

}
void MainWindow::SetGraphWidget (itPlot it1, itPlot it2) {


    if (graphWin != 0) delete graphWin;
    graphWin = new GraphWin(this,dataInfo,it1,it2,indNormalization);
    setCentralWidget ( graphWin );
}





