cZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ
	Subroutine takeoff(dep,delta,vpsource,take_off_ang,ierr,err_txt)
cZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ
c	Program calculates the take off angle (mesured in degrees) for 
c	given source depth (dep, km), epicentral distance (delta, deg.),
c	and Vp (vpsource, km/s) at source location. 
c
c	If vpsource=0 or dep > 70 km, the value of Vp at source location 
c	will be taken as for ak135 model (continental model). 
c
c	If given value of vpsource is larger then 7.5 and dtdh>0 - P wave 
c	for ak135 model propagates up (for distances > 3 degrees it can 
c	take place if dep>35 km), then given value for vpsource will be 
c	ignored and take off angle will be calculated using ak135 model 
c	value for Vp at the source location.
c
c	If dep <= 70 km and given value of vpsource is too large the value 
c	of cosine of emergency angle can become larger then 1. In this 
c	case the value of vpsource will be taken equal to 8.04 km/s - the 
c	value of Vp beneath the Moho boundary (see Subroutine EMERGE).
c 
c	Results: 
c	take off angle (take_off_ang, deg.). 
c	ierr - error index.
c	if ierr=0 - normal exit; 
c	if ierr=1 - following message is in err_txt: 
c	'No P arrivals for source depth X and distance Y',
c	where X=dep, and Y=delta; 
c	if ierr=2 - following message is in err_txt:
c	'Vp=a   h=b   cos(emergency angle)=c, 
c	where a - Vp value at the source location from ak135 model, 
c	b - source depth, and c - calculated value of emergency angle 
c	ixceeding 1 which indicates an error in original IASPEI program. 
c
c	ATTENTION! The program is based on ak135 model, and results
c	are valid for 20<=delta<=90 only!
c----------------------------------------------------------------------
      parameter (max=60)
	character*8 phcd(max)
	character*80 err_txt
      real*4 tt(max),dtdd(max),dtdh(max),mn(max),ts(max), ang(max)
	real*4 dep,delta,vpsource,take_off_ang
	integer ierr
	call ttimes_on
	ivploc=1
		if(vpsource.lt.1.e-6.or.dep.gt.70) then
		call vploc(dep,vpsource)
		ivploc=0
		end if
	ierr=0
	call ttimes(dep,delta,ierr,n,phcd,tt,mn,ts,dtdd,dtdh)
				if(ierr.ne.0) then	
	write(err_txt,100)dep,delta
100   format(1x,'No P arrivals for depth =',f7.2,
     *			',    delta =',f7.2,'@')
				RETURN
				end if
c-----------------------------------------------------------------------------
c	If vpsource value is given, dtdh>0 - P wave for ak135 propagates up (for
c	delta > 3 degrees it can take place if dep>35 km), and vpsource>7.5 
c	(the source is under the Moho), then given value for vpsource will be 
c	ignored and take off angle will be calculated using ak135 model value 
c	for Vp at the source location.
c-----------------------------------------------------------------------------
			if(ivploc.eq.1.and.dtdh(1).gt.0.and.vpsource.gt.7.5)
     *				call vploc(dep,vpsource)
	ierr=0
	call emerge(vpsource,dep,dtdd(1),dtdh(1),emer_ang,ierr,err_txt)
			if(ierr.ne.0) then 	
	take_off_ang=999.
			RETURN
			end if
c-----------------------------------------------------------------------------
c	If vpsource value is given, P wave for ak135 propagates up (dtdh>0), and 
c	vpsource<=7.5 (the source is above the Moho which is lower then 35km for
c	ak135 model) then the wave is supposed to propagate down (delta>3 deg.!)
c	end calculated for ak135 value of emergency angle is multiplied by -1.
c-----------------------------------------------------------------------------
			if(ivploc.eq.1.and.dtdh(1).gt.0.and.vpsource.lt.7.5)
     *						emer_ang=-emer_ang
	take_off_ang=90.-emer_ang
	call ttimes_off
	RETURN
	END
cZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ
	Subroutine vploc(dep,vpsource)
cZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ
	open(11,file='ak135.loc')
		read(11,*)z1,v1
		do i=1,10000
		read(11,*)z2,v2
			if(dep.gt.z2) then
			z1=z2
			v1=v2
			else 
			go to 1
			end if
		end do
1		continue
		vpsource=v1+(v2-v1)*(dep-z1)/(z2-z1)
	close(11)
					RETURN
					END
cZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ
	Subroutine emerge(vpsource,dep,dtdd,dtdh,emer_ang,ierr,err_txt)
cZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ
	character*80 err_txt
	integer ierr
	pi=3.1415927
	cosang=dtdd*vpsource*180./pi/(6371.-dep)
		if(abs(cosang).gt.1.) then
			if(abs(cosang).le.1.001) then
			cosang=1.
			else
				if(dep.gt.70) then
	ierr=2
	write(err_txt,100)vpsource,dep,cosang
100   format('Vp =',f7.2,'    h =',f7.2,
     *              '    cos(emergency angle) =',f7.2,'@')
	emer_ang=999.
			RETURN
				else
				vpsource=8.04
				cosang=dtdd*vpsource*180./pi/(6371.-dep)
					if(abs(cosang).gt.1.) then
						if(abs(cosang).le.1.001) then
						cosang=1.
						else
						ierr=2
						write(err_txt,100)vpsource,dep,cosang
						RETURN
						end if
					end if
				end if
			end if
		end if
	emer_ang=acos(abs(cosang))*180./pi
	emer_ang=-sign(emer_ang,dtdh)
					RETURN
					END
cZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ
	Subroutine ttimes(zs,delta,ierr,n,phcd,tt,mn,ts,dtdd,dtdh)
cZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ
      save
      parameter (max=60)
      logical log,prnt(3)
      character*8 phcd(max),phlst(10)
      character*20 modnam
      dimension tt(max),dtdd(max),dtdh(max),dddp(max),mn(max),ts(max)
      dimension usrc(2)
	integer ierr
      data in/1/,phlst(1)/'query'/,prnt(3)/.true./
c
c      write(6,*) 'enter model name:'
c      read(5,*) modnam
c      modnam='ak135'
c      write(6,*) 'This routine for calculating travel times for'
c      write(6,*) 'specific distances uses a set of precalculated'
c      write(6,*) 'tau-p tables for the',modnam,' model stored as'
c      write(6,*) modnam,'.hed  ',modnam,'.tbl'
c      write(6,*)
c      prnt(1) = .false.
c      prnt(2) = .false.
c      call assign(10,2,'ttim1.lis')
c      call tabin(in,modnam)
c      write(6,*) 'The source depth has to be specified and also'
c      write(6,*) 'the phase codes or keywords for the required branches'
c      write(6,*) 'ALL will give all available branches'
c      write(6,*) 'P  gives P-up,P,Pdiff,PKP, and PKiKP'
c      write(6,*) 'P+ gives P-up,P,Pdiff,PKP,PKiKP,PcP,pP,pPdiff,pPKP,' 
c      write(6,*) '         pPKiKP,sP,sPdiff,sPKP, and sPKiKP'
c      write(6,*) 'S  gives S-up,S,Sdiff, and SKS'
c      write(6,*) 'S+ gives S-up,S,Sdiff,SKS,sS,sSdiff,sSKS,pS,pSdiff,'
c      write(6,*) '         and pSKS '
c      write(6,*) 'basic gives P+ and S+ as well as '
c      write(6,*) '         ScP, SKP, PKKP, SKKP, PP, and PKPPKP '
c      write(6,*)
c      write(6,*) 'or give a generic phase name'
c      write(6,*)
c      write(6,*) 'You will have to enter a distance,'
c      write(6,*) 'if this is negative a new depth is calculated'
c      write(6,*) 'TO EXIT: give negative depth'
c      write(6,*)
c      call brnset(1,phlst,prnt)
c      call brnsetP(1,phlst,prnt)
c                                    choose source depth
c 3    call query('Source depth (km):',log)
c      read(*,*)zs
c      if(zs.lt.0.) go to 13
	 call depset(zs,usrc)
c                                    loop on delta
c 1    write(*,*)
c      call query('Enter delta:',log)
c      read(*,*)delta
c      if(delta.lt.0.) go to 3
c      write(6,*)
c     %'  delta    # code       time(s)    (min s)    dT/dD',
c     %'     dT/dh      d2T/dD2'
      call trtm(delta,max,n,tt,dtdd,dtdh,dddp,phcd)
      if(n.le.0) go to 2
      do 4 i=1,n
        mn(i)=int(tt(i)/60.)
        ts(i)=amod(tt(i),60.)
 4    continue
c
c      write(*,100)delta,(i,phcd(i),tt(i),mn(i),ts(i),dtdd(i),dtdh(i),
c     1 dddp(i),i=1,n)
c 100  format(/1x,f6.2,i5,2x,a,f9.2,i4,f7.2,f11.4,1p2e11.2/
c     1 (7x,i5,2x,a,0pf9.2,i4,f7.2,f11.4,1p2e11.2))
c      go to 1
      go to 13
c 2    write(*,101)delta
 2 	ierr=1
c 101  format(/1x,'No arrivals for delta =',f7.2)
c      go to 1
c                                    end delta loop
13				RETURN

	ENTRY ttimes_on
	modnam='ak135'
      prnt(1) = .false.
      prnt(2) = .false.
      call assign(10,2,'ttim1.lis')
      call tabin(in,modnam)
      call brnsetP(1,phlst,prnt)
	RETURN

	ENTRY ttimes_off
      call retrns(in)
      call retrns(10)
c      call exit(0)
					RETURN
	END
