//
// C++ Implementation: grid
//
// Description:
//
//
// Author: fmt <fmt@localhost.localdomain>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <QtGui>
#include <QMessageBox>
#include <QProcess>
#include "default.h"
#include "document.h"
#include "mainwindow.h"
#include "projectwizard.h"
#include "grid.h"
#include "editline.h"

GridImpl::GridImpl(QWidget *parent,MDocument* doc)
        : QDialog(parent),doc(doc) {
    setFont( parent->font() );
    setWindowIcon(QIcon(":/images/icon.png"));
    setupUi(this);
    InitDialog();
    SetDocumentToDialog();

}
void GridImpl::FinishWizard() {
    GetDocumentFromDialog();
    foreach (PropertyValue* val, values) val->SetValueToResetvalue();
}

void GridImpl::InitDialog() {

    values.append(doc->GetGridDepth());
    values.append(doc->GetGridDepthStep());
    values.append(doc->GetDepthPoint());

    values.append(doc->GetGridDip());
    values.append(doc->GetGridDipStep());
    values.append(doc->GetDipPoint());

    values.append(doc->GetGridStrike());
    values.append(doc->GetGridStrikeStep());
    values.append(doc->GetStrikePoint());

    values.append(doc->GetGridSlip());
    values.append(doc->GetGridSlipStep());
    values.append(doc->GetSlipPoint());

///------------------------- GridDepth -------------------------------------------------

    propDocLine.append(new LineDocProperty(lineEditStartDepth, doc->GetGridDepth()));

    doubleSpinBoxStepDepth->setDecimals (doc->GetGridDepthStep()->GetDecimal());
    doubleSpinBoxStepDepth->setMinimum (doc->GetGridDepthStep()->GetMin());
    doubleSpinBoxStepDepth->setMaximum (doc->GetGridDepthStep()->GetMax());
    doubleSpinBoxStepDepth->setSingleStep (1.0);
    doubleSpinBoxStepDepth->setSuffix(" km.");
    doubleSpinBoxStepDepth->setValue(doc->GetGridDepthStep()->GetValue().toFloat());

    propDocLine.append ( new LineDocProperty ( spinBoxPointsDepth, doc->GetDepthPoint() ) );

///------------------------- GridDip -------------------------------------------------

    propDocLine.append(new LineDocProperty(lineEditStartDip, doc->GetGridDip()));

    doubleSpinBoxStepDip->setDecimals (doc->GetGridDipStep()->GetDecimal());
    doubleSpinBoxStepDip->setMinimum (doc->GetGridDipStep()->GetMin());
    doubleSpinBoxStepDip->setMaximum (doc->GetGridDipStep()->GetMax());
    doubleSpinBoxStepDip->setSingleStep (1.0);
    doubleSpinBoxStepDip->setSuffix(" deg.");
    doubleSpinBoxStepDip->setValue(doc->GetGridDipStep()->GetValue().toFloat());

    propDocLine.append ( new LineDocProperty ( spinBoxPointsDip, doc->GetDipPoint() ) );

///------------------------- GridStrike -------------------------------------------------

    propDocLine.append(new LineDocProperty(lineEditStartStrike, doc->GetGridStrike()));

    doubleSpinBoxStepStrike->setDecimals (doc->GetGridStrikeStep()->GetDecimal());
    doubleSpinBoxStepStrike->setMinimum (doc->GetGridStrikeStep()->GetMin());
    doubleSpinBoxStepStrike->setMaximum (doc->GetGridStrikeStep()->GetMax());
    doubleSpinBoxStepStrike->setSingleStep (1.0);
    doubleSpinBoxStepStrike->setSuffix(" deg.");
    doubleSpinBoxStepStrike->setValue(doc->GetGridStrikeStep()->GetValue().toFloat());

    propDocLine.append ( new LineDocProperty ( spinBoxPointsStrike, doc->GetStrikePoint() ) );

///------------------------- GridSlip -------------------------------------------------

    propDocLine.append(new LineDocProperty(lineEditStartSlip, doc->GetGridSlip()));

    doubleSpinBoxStepSlip->setDecimals (doc->GetGridSlipStep()->GetDecimal());
    doubleSpinBoxStepSlip->setMinimum (doc->GetGridSlipStep()->GetMin());
    doubleSpinBoxStepSlip->setMaximum (doc->GetGridSlipStep()->GetMax());
    doubleSpinBoxStepSlip->setSingleStep (1.0);
    doubleSpinBoxStepSlip->setSuffix(" deg.");
    doubleSpinBoxStepSlip->setValue(doc->GetGridSlipStep()->GetValue().toFloat());

    propDocLine.append ( new LineDocProperty ( spinBoxPointsSlip, doc->GetSlipPoint() ) );




}
void GridImpl::SetDocumentToDialog() {
    foreach (LineDocProperty *docprop, propDocLine) docprop->SetValueFromDoc();

}
void GridImpl::GetDocumentFromDialog() {
//QMessageBox::critical(0,"", QString("GetDocumentFromDialog"));
///------------------------- GridDepth -------------------------------------------------
    doc->GetGridDepth()->SetValue(lineEditStartDepth->text());
    doc->GetGridDepthStep()->SetValue(QString("%1").arg(doubleSpinBoxStepDepth->value()));
    doc->GetDepthPoint()->SetValue(QString("%1").arg(spinBoxPointsDepth->value()));

///------------------------- GridDip -------------------------------------------------

    doc->GetGridDip()->SetValue(lineEditStartDip->text());
    doc->GetGridDipStep()->SetValue(QString("%1").arg(doubleSpinBoxStepDip->value()));
    doc->GetDipPoint()->SetValue(QString("%1").arg(spinBoxPointsDip->value()));

///------------------------- GridStrike -------------------------------------------------

    doc->GetGridStrike()->SetValue(lineEditStartStrike->text());
    doc->GetGridStrikeStep()->SetValue(QString("%1").arg(doubleSpinBoxStepStrike->value()));
    doc->GetStrikePoint()->SetValue(QString("%1").arg(spinBoxPointsStrike->value()));

///------------------------- GridSlip -------------------------------------------------

    doc->GetGridSlip()->SetValue(lineEditStartSlip->text());
    doc->GetGridSlipStep()->SetValue(QString("%1").arg(doubleSpinBoxStepSlip->value()));
    doc->GetSlipPoint()->SetValue(QString("%1").arg(spinBoxPointsSlip->value()));
}

bool GridImpl::CheckDialogParameters() {
    float start = lineEditStartDepth->text().toFloat();
    float step = doubleSpinBoxStepDepth->value();
    int npoint = spinBoxPointsDepth->value();
    float max = start + step*npoint;
    float limit = doc->GetGridDepth()->GetMax();
    if (max > limit) {
        QMessageBox::warning(0,mainWindowCaption(),
                             QString("The maximum depth (%1) greater than the depth limit %2 ")
                             .arg(max)
                             .arg(limit));
        SetPaletToGroupbox(groupBoxDepth,true);
        return false;
    } else SetPaletToGroupbox(groupBoxDepth,false);

///      dip
    start = lineEditStartDip->text().toFloat();
    step = doubleSpinBoxStepDip->value();
    npoint = spinBoxPointsDip->value();
    max = start + step*npoint;
    limit = doc->GetGridDip()->GetMax();
    if (max > limit) {
        QMessageBox::warning(0,mainWindowCaption(),
                             QString("The maximum dip (%1) greater than the dip limit %2 ")
                             .arg(max)
                             .arg(limit));
        SetPaletToGroupbox(groupBoxDip,true);
        return false;
    } else SetPaletToGroupbox(groupBoxDip,false);

/// strike
    start = lineEditStartStrike->text().toFloat();
    step = doubleSpinBoxStepStrike->value();
    npoint = spinBoxPointsStrike->value();
    max = start + step*npoint;
    limit = doc->GetGridStrike()->GetMax();
    if (max > limit) {
        QMessageBox::warning(0,mainWindowCaption(),
                             QString("The maximum strike (%1) greater than the strike limit %2 ")
                             .arg(max)
                             .arg(limit));
        SetPaletToGroupbox(groupBoxStrike,true);
        return false;
    } else SetPaletToGroupbox(groupBoxStrike,false);

/// slip
    start = lineEditStartSlip->text().toFloat();
    step = doubleSpinBoxStepSlip->value();
    npoint = spinBoxPointsSlip->value();
    max = start + step*npoint;
    limit = doc->GetGridSlip()->GetMax();
    if (max > limit) {
        QMessageBox::warning(0,mainWindowCaption(),
                             QString("The maximum slip (%1) greater than the slip limit %2 ")
                             .arg(max)
                             .arg(limit));
        SetPaletToGroupbox(groupBoxSlip,true);
        return false;
    } else SetPaletToGroupbox(groupBoxSlip,false);



    return true;
}

void GridImpl::SetPaletToGroupbox(QGroupBox * groupBox ,bool err) {
    QPalette palette;
    QBrush brush;
    if (err)
        brush.setColor(QColor(Qt::red));
    else
        brush.setColor(QColor(Qt::black));
    brush.setStyle(Qt::SolidPattern);
    palette.setBrush(QPalette::Active, QPalette::WindowText, brush);
    palette.setBrush(QPalette::Inactive, QPalette::WindowText, brush);
    /*
        QBrush brush1(QColor(119, 119, 119, 255));
        brush1.setStyle(Qt::SolidPattern);
        palette.setBrush(QPalette::Disabled, QPalette::WindowText, brush1);
    */
    groupBox->setPalette(palette);
}

