#include <QtGui>
#include <QPrinter>
#include <qwt_plot_printfilter.h>
#include "mainwindow.h"
#include "default.h"
#include "dataset.h"
#include "bode.h"
#include "bode_plot.h"

MainWindow *MainWindow::self = 0;
void delBodePlotAndNullify2 ( BodePlot* &pBodePlot ) {
    delete  pBodePlot;
    pBodePlot = 0;
}
MainWindow::~MainWindow() {
    if ( graphWin !=0 ) delete graphWin;
}
MainWindow::MainWindow ( QString &dirName ) : dataDirName ( dirName ) {
    self = this;
    graphWin = 0;
    setWindowTitle ( mainWindowCaption() );
    setWindowIcon ( QIcon ( ":/images/icon.png" ) );

    createActions();
    createMenus();
    createToolBars();
    createStatusBar();
    readSettings();
//QMessageBox::critical(0, "",dataDirName);


    loadProjectDirectory ( dataDirName );


}

void MainWindow::closeEvent ( QCloseEvent *event ) {

    writeSettings();
    event->accept();

}
void MainWindow::about() {
    QMessageBox::about ( this, tr ( "About Application" ),
                         tr ( "The <b>Application</b> example demonstrates how to "
                              "write modern GUI applications using Qt, with a menu bar, "
                              "toolbars, and a status bar." ) );
}


void MainWindow::createActions() {
    exitAct = new QAction ( QIcon ( ":/images/exit.png" ), tr ( "E&xit" ), this );
    exitAct->setShortcut ( tr ( "Ctrl+Q" ) );
    exitAct->setStatusTip ( tr ( "Exit the application" ) );
    connect ( exitAct, SIGNAL ( triggered() ), this, SLOT ( close() ) );



    zoominAct = new QAction ( QIcon ( ":/images/zoom-in.png" ), tr ( "Zoom &In" ),this );
    zoominAct->setShortcut ( tr ( "Ctrl++" ) );
    zoominAct->setStatusTip ( tr ( "Zoom in" ) );
    connect ( zoominAct, SIGNAL ( triggered() ),this, SLOT ( zoomin() ) );



    zoomoutAct = new QAction ( QIcon ( ":/images/zoom-out.png" ), tr ( "Zoom &Out" ),this );
    zoomoutAct->setShortcut ( tr ( "Ctrl+-" ) );
    zoomoutAct->setStatusTip ( tr ( "Zoom out" ) );
    connect ( zoomoutAct, SIGNAL ( triggered() ),this, SLOT ( zoomout() ) );

    printAct = new QAction ( QIcon ( ":/images/print.png" ), tr ( "&Print" ),this );
    printAct->setShortcut ( tr ( "Ctrl+P" ) );
    printAct->setStatusTip ( tr ( "Print depth curve" ) );
    connect ( printAct, SIGNAL ( triggered() ),this, SLOT ( print() ) );

    pdfAct = new QAction ( QIcon ( ":/images/pdf.png" ), tr ( "Save as p&df file" ),this );
    pdfAct->setShortcut ( tr ( "Ctrl+D" ) );
    pdfAct->setStatusTip ( tr ( "Save plots as p&df file" ) );
    connect ( pdfAct, SIGNAL ( triggered() ),this, SLOT ( pdf() ) );


    psAct = new QAction ( QIcon ( ":/images/printps.png" ), tr ( "Sav&e as ps file" ),this );
    psAct->setShortcut ( tr ( "Ctrl+E" ) );
    psAct->setStatusTip ( tr ( "Save plots as postscript file" ) );
    connect ( psAct, SIGNAL ( triggered() ),this, SLOT ( ps() ) );

    exportAct = new QAction ( QIcon ( ":/images/fileexport.png" ), tr ( "Save as &text file" ),this );
    exportAct->setShortcut ( tr ( "Ctrl+T" ) );
    exportAct->setStatusTip ( tr ( "Save depth curve as text file" ) );
    connect ( exportAct, SIGNAL ( triggered() ),this, SLOT ( exportData() ) );


    aboutAct = new QAction ( tr ( "&About" ), this );
    aboutAct->setStatusTip ( tr ( "Show the application's About box" ) );
    connect ( aboutAct, SIGNAL ( triggered() ), this, SLOT ( about() ) );

    aboutQtAct = new QAction ( tr ( "About &Qt" ), this );
    aboutQtAct->setStatusTip ( tr ( "Show the Qt library's About box" ) );
    connect ( aboutQtAct, SIGNAL ( triggered() ), qApp, SLOT ( aboutQt() ) );


}

void MainWindow::createMenus() {


    helpMenu = menuBar()->addMenu ( tr ( "&Help" ) );
    helpMenu->addAction ( printAct );
    helpMenu->addAction ( psAct );
    helpMenu->addAction ( pdfAct );
    //helpMenu->addAction(exportAct);
    helpMenu->addSeparator();
    helpMenu->addAction ( zoominAct );
    helpMenu->addAction ( zoomoutAct );
    helpMenu->addAction ( aboutAct );
    helpMenu->addAction ( aboutQtAct );
    helpMenu->addSeparator();
    helpMenu->addAction ( exitAct );
}

void MainWindow::createToolBars() {
    fileToolBar = addToolBar ( tr ( "File" ) );

    fileToolBar->addAction ( printAct );
    fileToolBar->addAction ( psAct );
    fileToolBar->addAction ( pdfAct );
    //fileToolBar->addAction(exportAct);

    fileToolBar->addSeparator ();

    fileToolBar->addAction ( zoominAct );
    fileToolBar->addAction ( zoomoutAct );
    fileToolBar->addSeparator ();

    fileToolBar->addSeparator ();
    fileToolBar->addAction ( exitAct );

}

void MainWindow::createStatusBar() {
    statusBar()->showMessage ( tr ( "Ready" ) );
}
void MainWindow::zoomin() {
    QFont f ( font() );
    f.setPointSize ( f.pointSize() + 1 );
    setFont ( f );
    if ( graphWin != 0 ) SetFontToGraph();
}
void MainWindow::zoomout() {
    QFont f ( font() );
    f.setPointSize ( f.pointSize() - 1 );
    setFont ( f );
    if ( graphWin != 0 ) SetFontToGraph();
}
void MainWindow::readSettings() {
    QSettings settings ( "mitp.ru", "MomTens Draw ObsCal" );
    QPoint pos = settings.value ( "pos", QPoint ( 100, 100 ) ).toPoint();
    QSize size = settings.value ( "size", QSize ( 400, 600 ) ).toSize();
    resize ( size );
    move ( pos );
    int fontPointSize = settings.value ( "FontPointSize", 10 ).toInt();
    QFont f ( font() );
    f.setPointSize ( fontPointSize );
    setFont ( f );
}

void MainWindow::writeSettings() {
    QSettings settings ( "mitp.ru", "MomTens Draw ObsCal" );
    settings.setValue ( "pos", pos() );
    settings.setValue ( "size", size() );
    settings.setValue ( "FontPointSize", font().pointSize() );
}

QString MainWindow::strippedName ( const QString &fullFileName ) {
    return QFileInfo ( fullFileName ).fileName();
}
const QString MainWindow::AbsDir ( const QString fileName ) const {

    if ( !fileName.isEmpty() ) return QFileInfo ( fileName ).absoluteDir ().absolutePath ();
    else return QDir::homePath ();

}

///==============================================================================

void MainWindow::PrintPrinterPainter ( QPrinter &printer,QwtPlotPrintFilter &filter ) {

    QPainter printerpainter;
    int heightPlot =printer.height();
    int widthPlot  = printer.width();
    int height =printer.height();
    int width = printer.width();
    vector<BodePlot* >::iterator  itPlotWin = graphWin->vBodePlot.begin();
    int graphWidth = ( *itPlotWin )->width();
    int graphHeight = ( *itPlotWin )->height();

    //qWarning ("XXX   PlotWinWidth %d   PlotWinHeight %d   PlotWidth  %d    PlotHeight  %d ",graphWidth,graphHeight,width,height);
    if ( !printerpainter.begin ( &printer ) )             // paint on printer
        return;
    vector<BodePlot* > vPrintPlot;

    vector<CDataSet* > dvector = dataInfo->GetDataSet();
    vector<CDataSet* >::iterator itta = dvector.begin();

    vPrintPlot.push_back ( new DurationPlot ( this, ( *itta ),dataInfo,"Plot Duration",true ) );
    itta++;

    vPrintPlot.push_back ( new MajorAxisPlot ( this, ( *itta ),dataInfo,"Plot Duration",true ) );
    itta++;

    vPrintPlot.push_back ( new MinorAxisPlot ( this, ( *itta ),dataInfo,"Plot Duration",true ) );
    itta++;

    vPrintPlot.push_back ( new CentroidPlot ( this, ( *itta ),dataInfo,"Plot Duration",true ) );
    itta++;

    vPrintPlot.push_back ( new MajorStrikePlot ( this, ( *itta ),dataInfo,"Plot Duration",true ) );
    itta++;

    vPrintPlot.push_back ( new VelocityStrikePlot ( this, ( *itta ),dataInfo,"Plot Duration",true ) );
    itta++;

    //////////////////////////////////////////////////////////////////////  !!!!!!!!!!!!!!!!!!!!!!!!      ////////////////////////////
    // Draw Page boudaries
    //  printerpainter.drawRect(0,0,widthPlot,heightPlot);
    float commonMargin = ( float ) 0.95;
    height = ( int ) ( commonMargin*printer.height() );


    QFont oldFont = printerpainter.font();
    QPen oldPen = printerpainter. pen();
// ----------------------------------- print header ------------------------------------------------------------------ //
    //printerpainter.drawRect(headerRect);
// ----------------------------------- print header ------------------------------------------------------------------ //

// ----------------------------------- print  footet -

    printerpainter.setFont ( oldFont );
    printerpainter.setPen ( oldPen );



    QRect rectMain ( 0,0,width,height );
    // Draw rect for Draw
    //printerpainter.drawRect(rectMain);

    int drawWigth = rectMain.width();
    int drawHeight = rectMain.height();

    int oldH = graphHeight;

    if ( numRow >1 ) {
        if ( ( graphHeight*numRow ) > drawHeight ) {
            graphHeight = ( int ) ( ( double ) drawHeight / numRow );
            graphWidth = ( int ) ( graphWidth * ( ( double ) graphHeight / oldH ) );
        }
    } else {
        if ( graphHeight > drawHeight ) {
            graphHeight =  drawHeight;
            graphWidth = ( int ) ( graphWidth * ( ( double ) graphHeight / oldH ) );
        }
    }

    int oldW = graphWidth;

    if ( numCol >1 ) {
        if ( ( graphWidth*numCol ) > drawWigth ) {
            graphWidth = ( int ) ( ( double ) drawWigth / numCol );
            graphHeight = ( int ) ( graphHeight * ( ( double ) graphWidth / oldW ) );
        }
    } else {
        if ( graphWidth > drawWigth ) {
            graphWidth =  drawWigth;
            graphHeight = ( int ) ( graphHeight * ( ( double ) graphWidth / oldW ) );
        }
    }


    int moving = ( int ) ( ( double ) ( drawWigth - graphWidth*numCol ) /2. );
    if ( moving > 0 )
        rectMain.moveTo ( moving,0 );
    int  deltayW = graphWidth;
    int  deltayH = graphHeight;

    //qWarning(" deltayW = %d deltayH = %d",deltayW,deltayH);

    QRect rect =  rectMain;

    rect.setWidth ( deltayW );
    rect.setHeight ( deltayH );

                int newX = (int)((widthPlot - deltayW*numCol)/2);
                int newY = (rectMain.height()-2*deltayH)/3;
    //    Draw set of graphs
    vector<BodePlot* >::iterator  it = vPrintPlot.begin();
    int j=0;
    int k=0;
    for ( j=0;j<numRow;j++ ) {
        for ( k=0;k<numCol;k++ ) {
            rect =  rectMain;
            rect.setWidth ( deltayW );
            rect.setHeight ( deltayH );
            rect.moveTo ( k*deltayW,j*deltayH);

            QRect r;

            r.setX ( newX+rect.x() + ( int ) ( rect.width() *0.01 ) );
            r.setY ( newY+rect.y() + ( int ) ( rect.height() *0.01 ) );
            r.setWidth ( ( int ) ( rect.width() *0.98 ) );
            r.setHeight ( ( int ) ( rect.height() *0.98 ) );

            printerpainter.drawRect ( r );

            ( *it )->print ( &printerpainter, r,filter );

            it++;
            if ( it == vPrintPlot.end() ) break;
        }

            rect =  rectMain;
            rect.setWidth ( deltayW );
            rect.setHeight ( deltayH );
            rect.moveTo (0,0);

            QRect r;

            r.setX (newX+ rect.x() + ( int ) ( rect.width() *0.01 ) );
            r.setY ( newY+rect.y() + ( int ) ( rect.height() *0.01 ) );
            r.setWidth ( ( int ) ( rect.width() *0.98 ) );
            r.setHeight ( ( int ) ( rect.height() *0.98 ) );

            printerpainter.drawRect ( r );
        if ( it == vPrintPlot.end() ) break;
    }


    for_each ( vPrintPlot.begin(),vPrintPlot.end(),delBodePlotAndNullify2 );
    vPrintPlot.erase ( remove ( vPrintPlot.begin(),vPrintPlot.end(),
                                static_cast<BodePlot* > ( 0 ) ),
                       vPrintPlot.end() );

    printerpainter.end();


}
///==============================================================================
void MainWindow::ps() {
    QString fileName = QFileDialog::getSaveFileName (
                           this, "Export File Name", userHomePath(),
                           "PostScript Documents (*.ps)" );
    if ( fileName.isEmpty() ) return;
//=============================================================================
    QString locatorStr = ".ps";
    int ind = fileName.length () - locatorStr.length ()-1;
    if ( ind < 0 ) ind =0;
    if ( ( fileName.indexOf ( locatorStr.toLower(),ind ) ) == -1 && ( fileName.indexOf ( locatorStr.toUpper(),ind ) ) == -1 ) {
        fileName.append ( locatorStr );
        if ( QFile::exists ( fileName ) ) {
            int ret = QMessageBox::warning ( this, mainWindowCaption(),
                                             QString ( "File %1 already exists.\n Do you want to overwrite it?" )
                                             .arg ( QDir::convertSeparators ( fileName ) ),
                                             QMessageBox::Yes | QMessageBox::Default,
                                             QMessageBox::No | QMessageBox::Escape );
            if ( ret == QMessageBox::No )
                return ;
        }
    }

    QPrinter printer;
    printer.setOutputFormat ( QPrinter::PostScriptFormat );
    printer.setOrientation ( QPrinter::Landscape );
    printer.setOutputFileName ( fileName );
    printer.setCreator ( "Bode example" );
    QwtPlotPrintFilter filter;
    if ( printer.colorMode() == QPrinter::GrayScale ) {
        int options = QwtPlotPrintFilter::PrintAll;
        options &= ~QwtPlotPrintFilter::PrintBackground;
        options |= QwtPlotPrintFilter::PrintFrameWithScales;
        filter.setOptions ( options );
    }

    PrintPrinterPainter ( printer,filter );

}
void MainWindow::pdf() {

    QString fileName = QFileDialog::getSaveFileName (
                           this, "Export File Name", userHomePath(),
                           "PDF Documents (*.pdf)" );
    if ( fileName.isEmpty() ) return;
//=============================================================================
    QString locatorStr = ".pdf";
    int ind = fileName.length () - locatorStr.length ()-1;
    if ( ind < 0 ) ind =0;
    if ( ( fileName.indexOf ( locatorStr.toLower(),ind ) ) == -1 && ( fileName.indexOf ( locatorStr.toUpper(),ind ) ) == -1 ) {
        fileName.append ( locatorStr );
        if ( QFile::exists ( fileName ) ) {
            int ret = QMessageBox::warning ( this, mainWindowCaption(),
                                             QString ( "File %1 already exists.\n Do you want to overwrite it?" )
                                             .arg ( QDir::convertSeparators ( fileName ) ),
                                             QMessageBox::Yes | QMessageBox::Default,
                                             QMessageBox::No | QMessageBox::Escape );
            if ( ret == QMessageBox::No )
                return ;
        }
    }
//=============================================================================

    QPrinter printer;
    printer.setOutputFormat ( QPrinter::PdfFormat );
    printer.setOrientation ( QPrinter::Landscape );
    printer.setOutputFileName ( fileName );
    printer.setCreator ( "Bode example" );
    QwtPlotPrintFilter filter;

    if ( printer.colorMode() == QPrinter::GrayScale ) {
        int options = QwtPlotPrintFilter::PrintAll;
        options &= ~QwtPlotPrintFilter::PrintBackground;
        options |= QwtPlotPrintFilter::PrintFrameWithScales;
        filter.setOptions ( options );
    }
    PrintPrinterPainter ( printer,filter );



}
void MainWindow::print() {
#if 0
    QPrinter printer;
#else
    QPrinter printer;//(QPrinter::HighResolution);

#endif
    printer.setOutputFormat ( QPrinter::NativeFormat );
    QString docName = "";//d_plot->title().text();
    if ( !docName.isEmpty() ) {
        docName.replace ( QRegExp ( QString::fromLatin1 ( "\n" ) ), tr ( " -- " ) );
        printer.setDocName ( docName );
    }

    printer.setCreator ( "Bode example" );
    printer.setOrientation ( QPrinter::Landscape );

#if QT_VERSION >= 0x040000
    QPrintDialog dialog ( &printer );
    if ( dialog.exec() ) {
#else
    if ( printer.setup() ) {
#endif

        QwtPlotPrintFilter filter;

        if ( printer.colorMode() == QPrinter::GrayScale ) {
            int options = QwtPlotPrintFilter::PrintAll;
            options &= ~QwtPlotPrintFilter::PrintBackground;
            options |= QwtPlotPrintFilter::PrintFrameWithScales;
            filter.setOptions ( options );
        }
        PrintPrinterPainter ( printer,filter );
    }




}

void MainWindow::exportData() {

}

void MainWindow::SetFontToGraph() {
    QFont f = font();
    vector<BodePlot* >bplot =  graphWin->vBodePlot;
    vector<BodePlot* >::iterator  it = graphWin->vBodePlot.begin();

    for ( it = bplot.begin(); it != bplot.end(); ++it ) ( *it )->ChangeFont ( f );


}


void MainWindow::loadProjectDirectory ( const QString &dirName ) {

    dataInfo = new CDataInfo ( dirName );
    configure();
//	project->PrintProject();
//dataInfo->PrintDataInfo();
}
void MainWindow::configure() {

    graphWin = 0;
    dvector = dataInfo->GetDataSet();
    numRow =2;
    numCol  = 3;
    numCurve = 6;
    SetGraphWidget ();
    SetFontToGraph();

}
void MainWindow::SetGraphWidget ( ) {
    if ( graphWin !=0 ) delete graphWin;

    graphWin = new GraphWin ( this,dataInfo,"GraphWindow" );

    setCentralWidget ( graphWin );

}





